#!/bin/bash

# Locate xmos_dir (for macOS)
detect_xmos_darwin() {
    local xmos_dir_prefix="XMOS_xTIMEcomposer*"

    # Try to find xTIMEcomposer with which
    local xcc="$(which xcc)"

    if [ x"${xcc}" == x"" ]; then
	# Try to find xTIMEcomposer with spotlite
	xcc="$(mdfind "(kMDItemDisplayName == xcc)" | grep XMOS | head  -n1)"
    fi

    if [ x"${xcc}" != x"" ] && ! ("${xcc}" --version | grep -q XMOS); then
	# it is not the good one
	xcc=
    fi
    
    if [ x"${xcc}" == x"" ]; then
	# Try to find xTIMEcomposer in common places
	xmos_dir="$(ls -d ~/Applications/${xmos_dir_prefix} ~/Applications/*/${xmos_dir_prefix} /Applications/${xmos_dir_prefix} /Applications/*/${xmos_dir_prefix} 2> /dev/null | head -n1)"
    else
	while [ -L "${xcc}" ]; do
	    xcc="$(readlink "${xcc}")"
	done

	xmos_dir="$(dirname "$(dirname "${xcc}")")"
    fi
}

# Locate xmos_dir (for Linux)
detect_xmos_linux() {
    local xmos_dir_prefix="XMOS/xTIMEcomposer/Community_*/"
    
    # Try to find xTIMEcomposer with which
    local xcc="$(which xcc)"
    
    if [ x"${xcc}" != x"" ] && ! ("${xcc}" --version | grep -q XMOS); then
	# it is not the good one
	xcc=
    fi
    
    if [ x"${xcc}" == x"" ]; then
	# Try to find xTIMEcomposer in common places
	# ~/usr, /usr or /opt when it is installed
	# ~/Downloads when it is not installed
	# / when it is in docker image
	# You can modify this part to suit your system
	xmos_dir="$(ls -d /${xmos_dir_prefix} ~/Downloads/${xmos_dir_prefix} ~/usr/${xmos_dir_prefix} /usr/local/${xmos_dir_prefix} /opt/${xmos_dir_prefix} 2> /dev/null | head -n1)"
    else
	xcc="$(readlink -f "${xcc}")"
	xmos_dir="$(dirname "$(dirname "${xcc}")")"
    fi
}

check_xmos_dir() {
    xmos_xpp=${xmos_dir}/libexec/xpp
    xmos_xcc=${xmos_dir}/libexec/xcc1llvm

    # Unset xmos_dir if none above are available
    if [ -x "${xmos_xpp}" ] && [ -x "${xmos_xcc}" ]; then
	true
    else
	xmos_dir=
    fi
}

help() {
    echo "Usage: xTIMEcomposer-config [OPTION]
Options:
  -h, --help        prints this help text to stdout
  --dir             print the path of xTIMEcomposer directory
  --xcc             prints the path of xcc
  --xpp             prints the path of xpp
 "
}

if [ "$(uname)" == "Darwin" ]; then
    detect_xmos_darwin
else
    detect_xmos_linux
fi
check_xmos_dir

if ( [ x"${xmos_dir}" == x"" ] ) ; then
    echo "xTIMEComposer not detected" >&2
    exit 1
fi

case ${1} in
    "--dir")
	printf "%q" "${xmos_dir}"
	;;
    "--xpp")
	printf "%q" "${xmos_xpp}"
	;;
    "--xcc")
	printf "%q" "${xmos_xcc}"
	;;
    "-h" | "--help")
	help
	;;
    *)
	help
	exit 1
	;;
esac


