/*
 * CiaoBlock.cpp
 *
 *  Created on: Jun 9, 2014
 *      Author: umer
 */

#include "CiaoBlock.h"

namespace LLVM2CIAO {



void CiaoBlock::clearLiterals()
{
	for (CiaoLiteral* lit : CiaoLiterals)
	{
		delete lit;
	}
	CiaoLiterals.clear();
}

CiaoBlock::CiaoBlock()
{
	llvmBlock=NULL;
	outputVars.clear();
}
CiaoBlock::CiaoBlock(BasicBlock * BB)
{
	outputVars.clear();
	setllvmBlock(BB);
}

CiaoBlock::~CiaoBlock()
{
	if(!llvmBlock || CiaoLiterals.size()>0)
	{
		clearLiterals();
	}
}

void CiaoBlock::addLiteral(CiaoLiteral * lit) //add before tail
{
	std::vector<CiaoLiteral*> ::iterator it=CiaoLiterals.end();
	it--;
	CiaoLiterals.insert(it,lit);
}
void CiaoBlock::addLiteralAtTail(CiaoLiteral * lit)
{
	std::vector<CiaoLiteral*> ::iterator it=CiaoLiterals.end();
	CiaoLiterals.insert(it,lit);
}
CiaoLiteral * CiaoBlock::removeLiteralAtTail()
{
	CiaoLiteral * result = CiaoLiterals.back();;
	CiaoLiterals.pop_back();
	return result;
}
void CiaoBlock::addLiteralAtHead(CiaoLiteral * lit)
{
	std::vector<CiaoLiteral*> ::iterator it=CiaoLiterals.begin();
	CiaoLiterals.insert(it,lit);
}
vector< pair<string,CiaoLiteral *> > CiaoBlock::removePhi(CiaoLiteral ** phiLit, int &phiLitIndex, string & newVar)
{
	vector< pair<string,CiaoLiteral *> > block2Lit;//=new vector< pair<string,CiaoLiteral *> >();

	for(int i=0;i<CiaoLiterals.size();i++)
	{
		//errs()<<"removePhi Here loop enter...\n";
		CiaoLiteral * lit = CiaoLiterals[i];
		if(lit->isPhi())
		{
			newVar = getNewVar();
			//errs()<<"removePhi Here if...\n";
			PHINode * phiInst = dyn_cast<PHINode>(lit->getllvmInstr());
			string blockName;
			CiaoLiteral * lit;

			for(int i=0; i<phiInst->getNumIncomingValues();i++)
			{
				blockName=prefixedBlockName(phiInst->getIncomingBlock(i));
				if(UndefValue * udefval = dyn_cast<UndefValue>(phiInst->getIncomingValue(i)))
				{
					string und="\'undef\'";
					lit=(new dummyLiteral(newVar, dummyLiteral::operation::Assignment, und,false));
				}
				else if(phiInst->getIncomingValue(i)->hasName())
				{
					lit=(new dummyLiteral(newVar, dummyLiteral::operation::Assignment, phiInst->getIncomingValue(i)->getName(),false));
				}
				else
				{ //Cases when operand does not have a name:
				  // 1) Constant operand
					if (ConstantInt *CI = dyn_cast<ConstantInt>(phiInst->getIncomingValue(i)))
						if (CI->getBitWidth()<=64) //Checking for integer bits
						{
							std::stringstream str;
							string s;
							str<<CI->getSExtValue();
							str>>s;
							lit=(new dummyLiteral(newVar, dummyLiteral::operation::Assignment, s,true));
						}
				}
				block2Lit.push_back(make_pair(blockName,lit));
			}

			//remove phi inst and push dummy inst
//				CiaoLiterals[i]=new dummyLiteral(phiInst->getName(),newVar,true);
//				delete lit;
			*phiLit=new dummyLiteral(phiInst->getName(), dummyLiteral::operation::Assignment, newVar,false);
			phiLitIndex=i;
			break;
		}
		else
		{
			//check here if lit is of type CiaoLiteral then break loop for speed up.
		}
	}
	return block2Lit;
}
bool CiaoBlock::replaceLiteral(int i, CiaoLiteral* lit)
{
	if(!(i<CiaoLiterals.size()))
		return false;
	CiaoLiteral * temp = CiaoLiterals[i];
	CiaoLiterals[i]=lit;
	delete temp;
	return true;

}
set<string> CiaoBlock::kill()
{
	set<string> killSet;
	for (CiaoLiteral* lit : CiaoLiterals)
	{
		set<string> defSet = lit->def();
		for (set<string>::iterator it=defSet.begin(); it!=defSet.end(); ++it)
			killSet.insert(*it);
	}

	return killSet;
}
set<string> CiaoBlock::gen()
{
	set<string> genSet;
	set<string> genSet_tmp;
	set<string> defSoFar;
	set<string>::iterator it;

	for (CiaoLiteral* lit : CiaoLiterals)
	{
		set<string> refSet = lit->ref();
		set<string> defSet = lit->def();

		set_difference(refSet.begin(), refSet.end(), defSoFar.begin(), defSoFar.end(), inserter(genSet_tmp, genSet_tmp.begin()));

		for (it=genSet_tmp.begin(); it!=genSet_tmp.end(); ++it)
			genSet.insert(*it);

		for (it=defSet.begin(); it!=defSet.end(); ++it)
			defSoFar.insert(*it);
	}

	return genSet;
}

vector<string> CiaoBlock::succ()
{
	vector<string> succList;
	Instruction * I = llvmBlock->getTerminator();

	if(isa<BranchInst>(I))
	{
		BranchInst * K = dyn_cast<BranchInst>(I);
		//errs()<<"";//errs()<<" "<<K->getNumSuccessors()<<" ";
		for(unsigned int i = 0; i<K->getNumSuccessors();i++)
			succList.push_back(prefixedBlockName(K->getSuccessor(i)));
	}
	return succList;
}
string CiaoBlock::getJumpVar()
{
	Instruction * I = llvmBlock->getTerminator();

	string jumpVar="";
	if(isa<BranchInst>(I))
	{
		BranchInst * K = dyn_cast<BranchInst>(I);
		if(K->getNumSuccessors()>1)
			jumpVar=K->getOperand(0)->getName();
	}
	return jumpVar;
}
void CiaoBlock::addinLive(set<string> inlive)
{
	for(string s:inlive)
		inLive.insert(s);

}
void CiaoBlock::setinLive(set<string> il)
{
	inLive=il;
}
set<string> CiaoBlock::getinLive()
{
	return inLive;
}
void CiaoBlock::setoutputVars(set<string> ov)
{
	outputVars=ov;
}
void CiaoBlock::insert_outputVar(string var)
{
	outputVars.insert(var);
}
set<string> CiaoBlock::getoutputVars()
{
	return outputVars;
}
void CiaoBlock::setoutLive(set<string> il)
{
	outLive=il;
}
set<string> CiaoBlock::getoutLive()
{
	return outLive;
}

string CiaoBlock::getName()
{
	return prefixedBlockName(llvmBlock);
}
void CiaoBlock::setllvmBlock(BasicBlock* BB)
{
	if(!llvmBlock)
		clearLiterals();

	llvmBlock=BB;
	//populate CiaoLiterals
	for (BasicBlock::iterator I = BB->begin(), E = BB->end(); I != E; ++I)
	{
		CiaoLiterals.push_back(new CiaoLiteral((Instruction*)I));
	}
}

string CiaoBlock::getMergedBlockName()
{
	string bname="";
	if(llvmBlock->hasName())
	{
		string name = getName();
		if(mergedBlocksNames.find(name)!=mergedBlocksNames.end() )
		{
			bname=transformBlockName(mergedBlocksNames[name].first);
//			inputArgs=mergedBlocksNames[name].second;
		}
		else if(name==prefixedBlockName(&llvmBlock->getParent()->getEntryBlock()))  //temporary hack, will not work if the entry block is named anything other than 'entry'
		{
			bname=name;
//			inputArgs=inLive;
		}
		else
		{
			errs()<<"\n ERROR in toClauseHead(). The block '"+name+"' does not exist in mergedBlocksNames \n";
		}
	}
	return bname;
}
set<string> CiaoBlock::getMergedBlockInputArgs()
{
	set<string> bArgs;
	if(llvmBlock->hasName())
	{
		string name = getName();
		if(mergedBlocksNames.find(name)!=mergedBlocksNames.end() )
			bArgs=mergedBlocksNames[name].second;
		else if(name==prefixedBlockName(&llvmBlock->getParent()->getEntryBlock()))  //temporary hack, will not work if the entry block is named anything other than 'entry'
			bArgs=inLive;
		else
			errs()<<"\n ERROR in toClauseHead(). The block '"+name+"' does not exist in mergedBlocksNames \n";
	}
	return bArgs;
}
string CiaoBlock::toClauseHead()
{
	string head=getMergedBlockName();
	set<string> inputArgs=getMergedBlockInputArgs();

	string args="";
	for (set<string>::iterator it=inputArgs.begin(); it!=inputArgs.end(); ++it)
	{
		string var=*it;
		args+=transformArg(var)+", ";
	}

	if(!outputVars.empty())//out
	{
		set<string>::iterator iter=outputVars.begin();
		if(hasRetStmt())
			args=transformArg(*iter)+", "+args; //assuming ret instruction returns only one argument.
		else
		{
			args="_RET_"+transformArg(*iter)+", "+args; //assuming ret instruction returns only one argument.
		}
	}

	if(!inputArgs.empty()||!outputVars.empty()/*out*/)
	{
		head+="("+args;
		head[head.size()-2]=')';
		head.resize(head.size()-1);
		head+=":-";
	}
	else
		head+=".";

	return head;
}
string CiaoBlock::getBlockCall()
{
	string callToThisBlock = toClauseHead();
	if(callToThisBlock[callToThisBlock.size()-1]=='-')
	{
		callToThisBlock.resize(callToThisBlock.size()-2);
	}
	else
		callToThisBlock.resize(callToThisBlock.size()-1);

	return callToThisBlock;
}
string CiaoBlock::toClauseBody()
{

	string body="";
	for(int i=0;i<CiaoLiterals.size();i++)
	{
		CiaoLiteral * lit = CiaoLiterals[i];
		if(i==CiaoLiterals.size()-1)
		{
			if(!hasRetStmt()&&!outputVars.empty())
			{//add output var
				set<string>::iterator iter=outputVars.begin();

				//errs()<<"CLASSNAME = "<<lit->myName()<<"\n\n\n";

				string br=lit->toLiteral();
				vector<std::string> br_parts=split(br, '(');
				string brLiteral=br_parts[0]+"(_RET_"+transformArg(*iter)+", "+br_parts[1];
				body+="\t"+brLiteral+".\n";

			}
			else
				body+="\t"+lit->toLiteral()+".\n";
		}
		else
			body+="\t"+lit->toLiteral()+",\n";
	}
	return body;
}
void CiaoBlock::print(raw_ostream &O)
{
	if(llvmBlock->hasName())
	{
		O<<":- trust comp "<<getMergedBlockName()<<"/"<<getMergedBlockInputArgs().size()+outputVars.size()<<" + (not_fails, is_det).\n";
		O<<toClauseHead()<<"\n";
	}

//	O<<"\t---IN-> ";
//	for (set<string>::iterator it=inLive.begin(); it!=inLive.end(); ++it)
//		O<<*it<<", ";
//
//	O<<"\n";
//
//	O<<"\t---OUT-> ";
//	for (set<string>::iterator it=outLive.begin(); it!=outLive.end(); ++it)
//		O<<*it<<", ";
//	O<<"\n";
//
//
//		O<<"\t---gen-> ";
//		set<string> gen=this->gen();
//		for (set<string>::iterator it=gen.begin(); it!=gen.end(); ++it)
//			O<<*it<<", ";
//
//		O<<"\n";
//
//		O<<"\t---kill-> ";
//		set<string> kill = this->kill();
//		for (set<string>::iterator it=kill.begin(); it!=kill.end(); ++it)
//			O<<*it<<", ";
//		O<<"\n";
//
//		O<<"\t---OUTPUTVARS-> ";
//		for (set<string>::iterator it=outputVars.begin(); it!=outputVars.end(); ++it)
//			O<<*it<<", ";
//		O<<"\n";

//		for (CiaoLiteral* lit : CiaoLiterals)
//		{
//			lit->print();
//		}
	O<<toClauseBody();

}
bool CiaoBlock::hasRetStmt()
{
//	if(llvmBlock->getTerminator()->getOpcode()==Instruction::Ret && llvmBlock->getTerminator()->getNumOperands()>0)
//		return true;
	if(!CiaoLiterals.empty())
	{
		CiaoLiteral * lastLit = CiaoLiterals[CiaoLiterals.size()-1];
		return lastLit->isRet();
	}
	return false;
}
string CiaoBlock::getRetStmtVar()
{
	string retVar;
//	if(llvmBlock->getTerminator()->getNumOperands()>0)
//		retVar=llvmBlock->getTerminator()->getOperand(0)->getName();
	if(!CiaoLiterals.empty())
	{
		CiaoLiteral * lastLit = CiaoLiterals[CiaoLiterals.size()-1];
		return lastLit->getRetStmtVar();
	}

	return retVar;
}
};
