/*
 * CiaoFunction.cpp
 *
 *  Created on: Jun 9, 2014
 *      Author: umer
 */


#include "CiaoFunction.h"

namespace LLVM2CIAO {


CiaoFunction::CiaoFunction()
{
	llvmFunction=NULL;
}
CiaoFunction::CiaoFunction(Function * FF, LoopInfo * LI)
{
	this->LI = LI;
	setllvmFunction(FF);
	//LI = &(MP->getAnalysis<LoopInfo>(FF));
}

CiaoFunction::~CiaoFunction()
{
	if(!llvmFunction || ciaoBlocks.size()>0)
	{
		clearBlocks();
	}
}

//FOR all blocks B DO
//  IN(B) = Gen(B)
//ENDFOR
//WHILE there aer changes DO
//  FOR each block B DO
//    Out(B) = \union In(S)
//        S  \in Succ(B)
//    In(B) = (Out(B) - Kill(B))  \union Gen(B)
//  ENDFOR
//ENDWHILE

void CiaoFunction::phiTransform()
{
	map<std::string, std::string> blockPair2phiVar;

	vector< pair<string,CiaoLiteral *> > * block2Lit;

	for (auto iter : ciaoBlocks)
	{
		CiaoBlock* block= iter.second;
		string b1,b2;
		CiaoLiteral *lit1=NULL;
		CiaoLiteral *lit2=NULL;
		CiaoLiteral *phiLit=NULL;
		int phiLitIndex=-1;
		string newVar;
		while(true) //keep removing first phi from block and returns two dummy clits to be added to predecessor blocks
		{
			vector< pair<string,CiaoLiteral *> > block2Lit = block->removePhi(&phiLit, phiLitIndex, newVar);
			if(block2Lit.empty())
				break;

			// Generate key of sorted blocknames+literal_values in order to avoid creating a new variable while removing phi nodes with exactly the same argument.
			std::vector<string> arr;
			for(int i = 0; i<block2Lit.size();i++)
				arr.push_back(string(block2Lit[i].first)+string(((dummyLiteral*)block2Lit[i].second)->getval()));
			std::sort(arr.begin(), arr.end());
			string stringKey;
			for(int i = 0; i<block2Lit.size();i++)
				stringKey+=arr[i];

			//Check a phi instruction with same arguments have not been processed before
			if(blockPair2phiVar.find(stringKey)==blockPair2phiVar.end())
			{
				blockPair2phiVar[stringKey]=newVar;
				for(int i=0;i<block2Lit.size();i++)
				{
					CiaoBlock * B = ciaoBlocks[block2Lit[i].first];
					B->addLiteral(block2Lit[i].second);
				}
			}
			else // already processed a phiNode that contains same values.
			{
				newVar=blockPair2phiVar[stringKey];

				for(int i=0;i<block2Lit.size();i++)
					delete block2Lit[i].second;
				block2Lit.clear(); // Delete contents too
			}

			((dummyLiteral*)phiLit)->setval(newVar);

			// remove phi literal with dummy literal with assignment.
			if(!(block->replaceLiteral(phiLitIndex, phiLit)))
				errs()<<"\n ERROR: Replace literal fail\n";
		}
	}
	blockPair2phiVar.clear();
}

bool CiaoFunction::hasRetStmt()
{
	for (auto iter : ciaoBlocks)
		if(iter.second->hasRetStmt())
			return true;
	return false;
}
void CiaoFunction::outputVars()
{
	bool flag=true;
	for (auto iter : ciaoBlocks)
	{
		CiaoBlock * block=iter.second;
		if(block->hasRetStmt())
		{
			block->insert_outputVar(block->getRetStmtVar());
			flag=false;
		}
	}
	if(flag)
		return;

	//WHILE there aer changes DO
	flag=true;
	while(flag)
	{
		//FOR each block B DO
		flag=false;
		for (auto iter : ciaoBlocks)
		{
			CiaoBlock * block=iter.second;
			set<string> outVars= block->getoutputVars();
			vector<string> succs_names = block->succ();
			for(string succ_name : succs_names)
			{
				CiaoBlock * succ_block = ciaoBlocks[succ_name];
				set<string> succ_block_outvars = succ_block->getoutputVars();

				for (set<string>::iterator it=succ_block_outvars.begin(); it!=succ_block_outvars.end(); ++it)
					outVars.insert(*it);
			}
			if(outVars.size()>1)
				errs()<<"\n WARNING: CiaoFunction::outputVars() multiple return statements in function\n";
			if( !(outVars==block->getoutputVars()) )
				flag=true;
			block->setoutputVars(outVars);
		}
	}
}
void CiaoFunction::liveVars()
{
	//FOR all blocks B DO
	//  IN(B) = Gen(B)
	//ENDFOR
	for (auto iter : ciaoBlocks)
	{
		CiaoBlock * block=iter.second;
		block->setinLive(block->gen());
	}

	bool flag=true;

	//WHILE there aer changes DO
	while(flag)
	{
		//FOR each block B DO
		flag=false;
		for (auto iter : ciaoBlocks)
		{
			CiaoBlock * block=iter.second;
			set<string> outLive = block->getoutLive();
			set<string> inLive = block->getinLive();
			set<string> gen_block=block->gen();
			set<string> kill_block=block->kill();

			//Out(B) = \union In(S)
			//  S  \in Succ(B)
			vector<string> succs_names = block->succ();
			for(string succ_name : succs_names)
			{
				CiaoBlock * succ_block = ciaoBlocks[succ_name];
				set<string> succ_block_inlive = succ_block->getinLive();

				for (set<string>::iterator it=succ_block_inlive.begin(); it!=succ_block_inlive.end(); ++it)
					outLive.insert(*it);
			}

			//block->setoutLive(outLive);

			//In(B) = (Out(B) - Kill(B))  \union Gen(B)
			set<string> out_kill_diff;
			set_difference(outLive.begin(), outLive.end(), kill_block.begin(), kill_block.end(), inserter(out_kill_diff, out_kill_diff.begin()));

			for (set<string>::iterator it=gen_block.begin(); it!=gen_block.end(); ++it)
				out_kill_diff.insert(*it);

			if( !(inLive==out_kill_diff) || !(outLive==block->getoutLive()))
				flag=true;

			block->setinLive(out_kill_diff);
			block->setoutLive(outLive);

		}
		//this->print();

	}
}
Function * CiaoFunction::getllvmFunction()
{
	return llvmFunction;
}

void CiaoFunction::setllvmFunction(Function* FF)
{
	if(!llvmFunction)
		clearBlocks();

	llvmFunction=FF;
	//populate ciaoBlocks
	for (Function::iterator BB = FF->begin(), E = FF->end(); BB != E; ++BB)
	{
		ciaoBlocks[prefixedBlockName((BasicBlock*)BB)]=new CiaoBlock((BasicBlock*)BB);
	}
}
void CiaoFunction::populateMergedBlocksNames()
{
	for (auto iter : ciaoBlocks)
	{
		vector<string> succList= iter.second->succ();
		if(succList.size()>=1)
		{
			string block_key=succList[0];
			string newBlockName=block_key;
			set<string> input_args=ciaoBlocks[block_key]->getinLive();

//				set<string> output_args=ciaoBlocks[block_key]->getoutputVars();//out

			if(succList.size()==2)
			{
				string block_key1=succList[1];
				newBlockName+="_"+block_key1;

				set<string> inLive2nd=ciaoBlocks[block_key1]->getinLive();

				set<string> output_args1=ciaoBlocks[block_key1]->getoutputVars();//out

				for (set<string>::iterator it=inLive2nd.begin(); it!=inLive2nd.end(); ++it)
					input_args.insert(*it);

//					for (set<string>::iterator it=output_args1.begin(); it!=output_args1.end(); ++it)//out
//						input_args.insert(*it);

				mergedBlocksNames[block_key1].first=newBlockName;
				mergedBlocksNames[block_key1].second=input_args;
			}

//				for (set<string>::iterator it=output_args.begin(); it!=output_args.end(); ++it)//out
//					input_args.insert(*it);

			mergedBlocksNames[block_key].first=newBlockName;
			mergedBlocksNames[block_key].second=input_args;

		}
	}
}

//	string toPredicate()
//	{
//
//	}
void CiaoFunction::propagateGaurds2Branches()
{
	for (auto iter : ciaoBlocks)
	{
		CiaoBlock* parent= iter.second;;
		vector<string> succs=parent->succ();
		if(succs.size()>1)
		{
			string jumpVar = parent->getJumpVar();

			CiaoBlock* child = ciaoBlocks[succs[0]];
			CiaoLiteral * lit = new dummyLiteral(jumpVar,dummyLiteral::operation::Equal,"1",true);
			child->addLiteralAtHead(lit);

			child = ciaoBlocks[succs[1]];
			lit = new dummyLiteral(jumpVar,dummyLiteral::operation::NotEqual,"1",true);
			child->addLiteralAtHead(lit);
		}
	}
}
void CiaoFunction::transform()
{
	phiTransform();

	propagateGaurds2Branches();

	liveVars();

	outputVars();

	populateMergedBlocksNames();
	if(LoopTransformOption.getValue()=="yes")
		transformNestedLoops();
}
set<string> processedBlocks;
void CiaoFunction::transformNestedLoops_aux(Loop* L)
{

	if(!L)
		return;
	if(L->getSubLoops().size()==0)
		return;

	for(int i=0;i<L->getSubLoops().size();i++)
	{
		Loop * NL = (Loop*)(L->getSubLoops()[i]);
//		if(NL->getSubLoops().size()>0)
			transformNestedLoops_aux(NL);
	}

	string block_to_remove_lit_from=L->getSubLoops()[L->getSubLoops().size()-1]->getExitBlock()->getName(); //TODO: more than one loop exit.
	string block_to_add_lit_to=L->getSubLoops()[0]->getLoopPreheader()->getName();

	//ExitBlock2PreHeaderBlock: Remove subloop's exit block's jump (the last literal) and add it to the loop's preheader block (at the tail).
	ciaoBlocks[getName()+"_"+block_to_add_lit_to]->addLiteralAtTail(ciaoBlocks[getName()+"_"+block_to_remove_lit_from]->removeLiteralAtTail());

	//string parentHeader = L->getName();
	string parentHeader = ciaoBlocks[getName()+"_"+block_to_remove_lit_from]->succ()[0];
	set<string> parentInLive=ciaoBlocks[parentHeader]->getinLive();

	for(int i=0;i<L->getSubLoops().size();i++)
	{
		Loop * NL = (Loop*)(L->getSubLoops()[i]);
		for(BasicBlock* b : NL->getBlocks())
		{
			if(processedBlocks.find(string(b->getName()))!=processedBlocks.end())
				continue;
			processedBlocks.insert(string(b->getName()));

			string blockName = getName()+"_"+string(b->getName());
			string mergedName=mergedBlocksNames[blockName].first;


			for(string s : parentInLive)
				mergedBlocksNames[blockName].second.insert(s);

			if(mergedName.size()>blockName.size())
			{
				vector<string> tokens = split(mergedName, '_');
				for(string tok : tokens)
				{
					if(tok!=string(getName())&&tok!=string(b->getName())&&tok.size()>0)
					{
						processedBlocks.insert(tok);
						for(string s : parentInLive)
							 mergedBlocksNames[getName()+"_"+tok].second.insert(s);
						break;
					}

				}
			}

		}
	}

}

void CiaoFunction::transformNestedLoops()
{
	processedBlocks.clear();
	for(Loop* L : *LI)
	{
		transformNestedLoops_aux(L);
	}
}

set<string> CiaoFunction::getOutputArgs()
{
	set<string> outvars;
	if(llvmFunction)
		outvars = ciaoBlocks[prefixedBlockName(&llvmFunction->getEntryBlock())]->getoutputVars();
	return outvars;
}
set<string> CiaoFunction::getInputArgs()
{
	std::set<string> inArgs;

	if(llvmFunction)
		for (Function::arg_iterator AI = llvmFunction->arg_begin(), AE = llvmFunction->arg_end();AI != AE; ++AI) //Function args
		{
			if(AI->hasName())
				inArgs.insert(AI->getName());
			else
				errs()<<"\n WARNING: In CiaoFunction:getInputArgs(), llvm function unnamed input argument.\n";
		}
	return inArgs;
}
string CiaoFunction::getName()
{
	string name="";
	if(llvmFunction)
		name = llvmFunction->getName();
	return name;
}

string CiaoFunction::getCiaoType(Type* type)
{
	Type* currType=type;
	string ciaoType="";
	int brackets2close=0;
	bool flag=true;
	while(flag)
	{
		switch(currType->getTypeID())
		{
			case Type::PointerTyID:
				ciaoType+="list(";
				brackets2close++;
				currType=currType->getContainedType(0);
				break;
			case Type::ArrayTyID:
				ciaoType+="list(";
				brackets2close++;
				currType=currType->getContainedType(0);
				break;
			case Type::FloatTyID:
//				ciaoType+="num";
//				flag=false;
//				break;
			case Type::DoubleTyID:
//				ciaoType+="num";
//				flag=false;
//				break;
			case Type::IntegerTyID:
				ciaoType+="num";
				flag=false;
				break;

			default:
				flag=false;
		};
	}
	while(brackets2close>0)
	{
		ciaoType+=")";
		brackets2close--;
	}

	return ciaoType;
}

string CiaoFunction::getEntryAssrt()
{
	string entry="";
	if(llvmFunction)
	{
		entry=":- entry "+getName()+"/"+std::to_string(getInputArgs().size()+getOutputArgs().size())+":";

		if(getOutputArgs().size()>0)
		{
			int n=getOutputArgs().size();
			while(n--)
				entry+="var*";
		}

		if(llvmFunction->getArgumentList().size()>0)
			for (Function::arg_iterator AI = llvmFunction->arg_begin(), AE = llvmFunction->arg_end();AI != AE; ++AI) //Function args
			{
				if(AI->hasName())
					entry+=getCiaoType(AI->getType())+"*";
				else
					errs()<<"\n WARNING: In CiaoFunction:getEntryAssrt(), llvm function unnamed input argument.\n";
			}

		entry[entry.size()-1]='.';
	}
	return entry;
}

string CiaoFunction::toFunctionHead()
{
	string head=getName();;
	set<string> inputArgs=getInputArgs();

	string args="";
	for (set<string>::iterator it=inputArgs.begin(); it!=inputArgs.end(); ++it)
	{
		string var=*it;
		args+=transformArg(var)+", ";
	}

	set<string> outputVars=getOutputArgs();

	if(!outputVars.empty())
	{
		set<string>::iterator iter=outputVars.begin();
		args="_RET_"+transformArg(*iter)+", "+args;
	}

	if(!inputArgs.empty()||!outputVars.empty())
	{
		head+="("+args;
		head[head.size()-2]=')';
		head.resize(head.size()-1);
		head+=":-";
	}
	else
		head+=":-";

	return head;
}
string CiaoFunction::callToEntryBlock()
{
	if(llvmFunction)
	{
		CiaoBlock * entryBlock = ciaoBlocks[prefixedBlockName(&llvmFunction->getEntryBlock())];
		return entryBlock->getBlockCall();
	}
	else
		errs()<<"\n ERROR: CiaoFunction::callToEntryBlock(). No Entry block.\n";
	return "";
}
void CiaoFunction::print(raw_ostream &O)
{
	transform();

	getEntryAssrt();

	string callToEntry=callToEntryBlock();

	//O<<":- entry "<<getName()<<"/"<<getInputArgs().size()+getOutputArgs().size()<<".\n";
	O<<getEntryAssrt()<<"\n";
	O<<":- export( "<<getName()<<"/"<<getInputArgs().size()+getOutputArgs().size()<<").\n";
	O<<toFunctionHead()<<"\n";
	O<<"\t"<<callToEntry<<".\n";
	for (auto iter : ciaoBlocks)
	{
		iter.second->print(O);
	}
	//getLoopTransformation();
}
std::set<string> CiaoFunction::sliceFunction(LoopInfo * LI)
{
	std::set<string> criterion = getLoopGuards(LI);

	// Temporary hack for fir benchmark, uncomment for other benchmarks.
//	criterion.insert("deref18");
//	criterion.insert("coeffs");
//	criterion.insert("deref19");

	while(true)
	{
		std::set<string> criterion_prev=criterion;
		accumulateCriterion(criterion);
		//fixpoint test
		if(criterion_prev==criterion)
			break;
	}
	return criterion;
}

std::set<string> CiaoFunction::accumulateCriterion(std::set<string> &crit)
{
//	std::set<string> union_crit=crit;
//	for (iplist<CiaoBlock>::iterator BB = BasicBlockList.begin(), E = BasicBlockList.end(); BB != E; ++BB)
//	{
//		BB->sliceClause(crit);
//	}
	return crit;
}

void CiaoFunction::separateNestedLoops(LoopInfo *LI)
{

	std::vector <pair<string, string> > v = getLoopTransformation();

	for(std::vector<pair<string, string> >::iterator it = v.begin();it < v.end(); ++it)
	{
		errs()<<"";//errs()<< it->first<<", "<<it->second;

//			CiaoBlock * cl1 = BlockName2CiaoClause[it->first];
//			CiaoBlock * cl2 = BlockName2CiaoClause[it->second];

				//BlockName2CiaoClause.lookup(it->first.c_str());
//			if(!cl1 || !cl2)
//				errs()<<"";//errs()<<"nulllllllll for " <<it->first;

		//errs()<<"";//errs()<<"***888****"<<->getBlockName();
//			ExitBlock2PreHeaderBlock(cl1, cl2 );
	}

}


void CiaoFunction::getNestedLoops(Loop *L, Loop *PL, std::vector <pair<string, string> > &BL)
{
	if(L==NULL)
		return;

	for(int i=0;i<L->getSubLoops().size();i++)
		getNestedLoops((Loop*)(L->getSubLoops()[i]), L, BL);

	if(PL!=NULL)
	{
		errs()<<"\n ==========>EXITBLOCK"<<prefixedBlockName(L->getExitBlock())<<", LOOP-PREHEADER"<<L->getLoopPreheader()->getName()<<") \n";
		pair<string, string> p;
		p.first = prefixedBlockName(L->getExitBlock());
		p.second = prefixedBlockName(L->getLoopPreheader());
		BL.push_back(p);
	}
}

std::vector <pair<string, string> > CiaoFunction::getLoopTransformation()
{
	std::vector <pair<string, string> > BasicBlockList;
	if(LI!=NULL)
		for (Loop *L : *LI)
			getNestedLoops(L, NULL, BasicBlockList);

	return BasicBlockList;
}

// Works upto two nestings of loops, needs to be implemented recursively
std::set<string> CiaoFunction::getLoopGuards(LoopInfo *LI)
{
	std::set<string> loopGuards;

	if(LI!=NULL)
		for (Loop *L : *LI)
		{
			//errs()<<"";//errs()<<";;;;;;;\n";
			//=======>error thrown in lookup.
			//CiaoBlock * cls = BasicBlock2CiaoClause.lookup((BasicBlock*)L->getHeader());
			//errs()<<"";//errs()<<";;;;;;;\n";


			loopGuards.insert(L->getHeader()->getTerminator()->getOperand(0)->getName());

			TerminatorInst * TI = L->getHeader()->getTerminator();

	//		if(*(TI->getOperand(0)))
	//			errs()<<"";//errs()<<"\n\n infinite loop detected \n\n ";

			if(L->hasDedicatedExits())
				errs()<<"";//errs()<<"\n\n infinite loop detected \n\n ";
			errs()<<"";//errs()<< "\n loop guard  =  "<<L->getHeader()->getTerminator()->getOperand(0)->getName();
			//errs()<<"";//errs()<<";;;;;;;\n";
			for(int i=0;i<L->getSubLoops().size();i++)
			{
				Loop * NL = (Loop*)(L->getSubLoops()[i]);

				//cls = BasicBlock2CiaoClause.lookup(NL->getHeader());
				loopGuards.insert(NL->getHeader()->getTerminator()->getOperand(0)->getName());
			}
		}

	return loopGuards;
}

}
