/*
 * CiaoFunction.h
 *
 *  Created on: Jun 9, 2014
 *      Author: umer
 */

#ifndef CIAOFUNCTION_H_
#define CIAOFUNCTION_H_

#include "llvm/IR/Instructions.h"
#include "llvm/Support/raw_ostream.h"
#include "llvm/IR/Constants.h"
#include "llvm/Analysis/LoopInfo.h"

#include "Globals.h"
#include "CiaoBlock.h"

#include <vector>
#include <set>

using namespace std;
using namespace llvm;


namespace LLVM2CIAO {

class CiaoFunction
{
	Function * llvmFunction;
	LoopInfo * LI;
	void clearBlocks()
	{
		for (auto iter : ciaoBlocks)
		{
			delete iter.second;
		}
		ciaoBlocks.clear();
	}
protected:
	map<std::string, CiaoBlock *> ciaoBlocks;
public:
	CiaoFunction();
	CiaoFunction(Function * FF, LoopInfo * L);
	~CiaoFunction();
	//FOR all blocks B DO
	//  IN(B) = Gen(B)
	//ENDFOR
	//WHILE there aer changes DO
	//  FOR each block B DO
	//    Out(B) = \union In(S)
	//        S  \in Succ(B)
	//    In(B) = (Out(B) - Kill(B))  \union Gen(B)
	//  ENDFOR
	//ENDWHILE
	void phiTransform();
	void outputVars();
	bool hasRetStmt();
	void liveVars();
	Function * getllvmFunction();
	void setllvmFunction(Function* FF);
	void populateMergedBlocksNames();
//	string toPredicate();
	void propagateGaurds2Branches();
	void transformNestedLoops();
	void transformNestedLoops_aux(Loop* );
	void transform();
	string getCiaoType(Type* type);
	set<string> getOutputArgs();
	string getEntryAssrt();
	set<string> getInputArgs();
	string getName();
	string toFunctionHead();
	string callToEntryBlock();
	void print(raw_ostream &O);
	std::set<string> sliceFunction(LoopInfo * LI);
	std::set<string> accumulateCriterion(std::set<string> &crit);
	void separateNestedLoops(LoopInfo *LI);
	void getNestedLoops(Loop *L, Loop *PL, std::vector <pair<string, string> > &BL);
	std::vector <pair<string, string> > getLoopTransformation();
	// Works upto two nestings of loops, needs to be implemented recursively
	std::set<string> getLoopGuards(LoopInfo *LI);
	friend class CiaoBlock;
};

}

#endif /* CIAOFUNCTION_H_ */


