/*
 * CiaoModule.cpp
 *
 *  Created on: Jul 6, 2017
 *      Author: umer
 */

#include "CiaoModule.h"

namespace LLVM2CIAO {

void CiaoModule::clearFunctions()
{
	for (auto iter : ciaoFunctions)
	{
		delete iter.second;
	}
	ciaoFunctions.clear();
}


CiaoModule::CiaoModule()
{
	llvmModule=NULL;
}
CiaoModule::CiaoModule(Module * MM, ModulePass * MP)
{
	setllvmModule(MM, MP);
}

CiaoModule::~CiaoModule()
{
	if(!llvmModule || ciaoFunctions.size()>0)
	{
		clearFunctions();
	}
}

Module * CiaoModule::getllvmModule()
{
	return llvmModule;
}

void CiaoModule::setllvmModule(Module* MM, ModulePass * MP)
{
	if(!llvmModule)
		clearFunctions();

	llvmModule=MM;


	//populate ciaoFunctions
	for (Module::iterator FF = MM->begin(), E = MM->end(); FF != E; ++FF)
	{
		if(!FF->isDeclaration()) //skips library calls e.g., printf(...);
		{
			LoopInfoWrapperPass * LIWP= &MP->getAnalysis<LoopInfoWrapperPass>(*FF);


			ciaoFunctions[FF->getName()]=new CiaoFunction((Function*)FF, &LIWP->getLoopInfo());
		}

	}


}
void CiaoModule::printAssertions(raw_ostream & OS)
{
	std::ifstream ifs;

	  ifs.open (AssertionFilename.c_str(), std::ifstream::in);

	  char c = ifs.get();

	  while (ifs.good()) {
		OS<<c;
		c = ifs.get();
	  }

	  ifs.close();
}
void CiaoModule::printPackages(raw_ostream & OS)
{
	std::ifstream ifs;

	  ifs.open (PackagesFilename.c_str(), std::ifstream::in);

	  char c = ifs.get();

	  while (ifs.good()) {
		OS<<c;
		c = ifs.get();
	  }

	  ifs.close();
}
void CiaoModule::print(raw_ostream &O)
{
	O<<":-module(_,[],[]).\n";
	printPackages(O);
	O<<"\n";
	printAssertions(O);
	O<<"\n";
	//errs()<<"\n"<<llvmModule->getName()<<"(";
	for (auto iter : ciaoFunctions)
	{
		iter.second->print(O);
	}
}



}
