/*
 * llvm2ciao.cpp
 *
 *  Created on: May 16, 2014
 *      Author: umer
 */
#include "llvm/Pass.h"
#include "llvm/IR/Function.h"
#include "llvm/IR/Module.h"
#include "llvm/IR/BasicBlock.h"
#include "llvm/IR/Instructions.h"
#include "llvm/Analysis/CFG.h"
#include "llvm/IR/Type.h"
#include "llvm/IR/Constants.h"
#include "llvm/Analysis/LoopInfo.h"
#include "llvm/ADT/MapVector.h"
#include "llvm/Support/CommandLine.h"
#include "llvm/Support/FileSystem.h"
//#include "llvm/Support/raw_ostream.h"
//#include "llvm/Support/FileSystem.h"
//#include "llvm_to_ciao.h"

//#include "Jzon.h"

//you may not need all of the following:
//#include "llvm/ADT/hash_map"  //provides STL hash_map
//#include "llvm/ADT/hash_set"
#include <vector>

#include <deque>
#include <set>
#include <algorithm>
#include <ostream>
#include <fstream>
#include <iostream>
#include <stdio.h>
#include <sstream>
#include <vector>
#include <list>
#include <algorithm>
#include <typeinfo>
#include <fstream>

#include "Globals.h"
#include "CiaoLiteral.h"
#include "CiaoBlock.h"
#include "CiaoFunction.h"
#include "CiaoModule.h"

using namespace std;
using namespace llvm;

namespace LLVM2CIAO
{


//static cl::opt<string> OutputFilename("of", cl::desc("Specify output filename"), cl::value_desc("filename"));
//static cl::opt<string> AssertionFilename("assrt", cl::desc("Specify frontend assertion filename"), cl::value_desc("filename"));
//static cl::opt<string> DefaultModel("default_model", cl::desc("Include default model"), cl::value_desc("include_defaul_model"));
//static cl::opt<string> PackagesFilename("pkg", cl::desc("Specify filename containing packages to be included in the transformed program"), cl::value_desc("filename"));


class llvm_to_ciao : public ModulePass//, public ilist_node<char>
{

	std::vector<unsigned> counts;
	//CiaoFunction * ciaoFunc;
	//map<std::string, CiaoFunction *> FuncName2CiaoFunc;
	 LoopInfo *LI;
public:
	static char ID;

	llvm_to_ciao() :
	ModulePass(ID) //(intptr_t)&ID
	{
		//ciaoFunc=new CiaoFunction();
	}

	~llvm_to_ciao()
	{
		// Bug pointer being freed was not allocated
		//delete ciaoFunc;
	}

	// We don't modify the program, so we preserve all analyses
	virtual void getAnalysisUsage(AnalysisUsage &AU) const
	{
		AU.setPreservesAll();
		AU.addPreserved<LoopInfoWrapperPass>();
		AU.addRequired<LoopInfoWrapperPass>();
		// Following can't be done since InstructionNamer is a transform pass.
		// Instruction namer has to be run before the current pass is run from the command line.
		// AU.addRequiredID(InstructionNamerID);
	}

	void set_operations()
	{
	// Data
	std::vector<string> a,b;
	a.push_back("umer");b.push_back("aafia");
	a.push_back("usman");b.push_back("faiqa");
	a.push_back("hira");b.push_back("faiza");
	a.push_back("abubakar");b.push_back("umer");

	// Allocate space for intersection
	std::vector<string> v(a.size()+b.size());

	// Sort as required by set_intersection
	std::sort(a.begin(),a.end());
	std::sort(b.begin(),b.end());

	// Union
	std::vector<string>::iterator it = std::set_union(a.begin(),a.end(),b.begin(),b.end(),v.begin());

	// Intersection
	//std::vector<string>::iterator it = std::set_union(a.begin(),a.end(),b.begin(),b.end(),v.begin());

	// Differnece
	//std::vector<string>::iterator it = std::set_difference(a.begin(),a.end(),b.begin(),b.end(),v.begin());

	// Display
	v.erase(it,v.end());
	for(std::vector<string>::iterator it = v.begin();it < v.end(); ++it)
		std::cout<<*it<<std::endl;
	}

	void getNestedLoops(Loop *L, Loop *PL, int depth)
	{
		if(L==NULL)
			return;

		for(int i=0;i<L->getSubLoops().size();i++)
			getNestedLoops((Loop*)(L->getSubLoops()[i]), L, depth+1);

		if(PL!=NULL)
		{
			errs()<<"";//errs()<<"\n ==========>"<<L->getExitBlock()->getName()<<", "<<L->getLoopPreheader()->getName()<<") \n";
		}
	}

	void printNestedLoops(Loop *L, Loop *PL, int depth)
	{
		if(L==NULL)
			return;

		for(int j=0;j<depth;j++)
			errs()<<"";//errs()<<"  ";

		errs()<<"";//errs()<<"Guard = " <<L->getHeader()->getTerminator()->getOperand(0)->getName();
		errs()<<"";//errs()<<"  Entry( " << L->getHeader()->getName()<<" ), ";

		// add continuation after the terminator instruction in preheader block
//		if(!(L->getLoopPreheader()))
			errs()<<"";//errs()<<"Preheader( " << L->getLoopPreheader()->getName()<<"), ";
		errs()<<"";//errs()<<"Latch( " << L->getLoopLatch()->getName()<<"), ";
/////////////////// Same as getLoopPreheader()////////////////////
//		if(PL!=NULL)
//		{
//			//SmallPtrSet<BasicBlock*, 4> BadPreds;
//			for (pred_iterator PI = pred_begin(L->getHeader()),
//				 PE = pred_end(L->getHeader()); PI != PE; ++PI)
//			{
//			  BasicBlock *P = *PI;
//			  if (PL->contains(P) & !L->contains(P))
//				//BadPreds.insert(P);
//				  errs()<<"";//errs()<<"FromBlock( " << P->getName()<<"), ";
//			}
//		}
/////////////////////////////////////////////////////////////////
		//if(L->gete)
//		errs()<<"";//errs()<<"Exit block: " << L->getExitBlock()->getName()<<"\n";

		SmallVector<BasicBlock*, 8> v;
		L->getExitBlocks(v);
		errs()<<"";//errs()<<"Exit( ";
		for(int k=0;k<v.size();k++)
		{
			errs() << v[k]->getName()<<", ";

			for (pred_iterator PI = pred_begin(v[k]),
				 PE = pred_end(v[k]); PI != PE; ++PI)
			{
			  BasicBlock *P = *PI;
			  if (L->contains(P))
			  {
				  errs()<<"";//errs()<<"-pred( " << P->getName()<<") ";
				  // Add call to v[k] at the end of L->getLoopPreheader block.
				  // Remove v[k] from its predecessor block P and replace it with a dummy return block.
			  }
			}
		}
		errs()<<"";//errs()<<" )\n\n";
		//L->get

		for(int i=0;i<L->getSubLoops().size();i++)
			printNestedLoops((Loop*)(L->getSubLoops()[i]), L, depth+1);
	}

	virtual bool runOnFunction1(Function &F)
	{
//		CiaoFunction * f = new CiaoFunction(&F);
//		f->print();
//		delete f;
		return false;
	}
	virtual bool runOnFunction_last(Function &F)
	{
		errs()<<"\n"<<F.getName()<<"(-";
		for (Function::arg_iterator AI = F.arg_begin(), AE = F.arg_end();AI != AE; ++AI)
		{
			if(AI->hasName())
			{
		  		errs()<<" "<<*(AI->getType() )<<" "<<AI->getName()<<AI->getType();
		    	errs()<<", ";
		    }
		}
		errs()<<")";

		//Iterate over the Function's Blocks
		//CiaoLiteral ciaoLit;
		for (Function::iterator BB = F.begin(), E = F.end(); BB != E; ++BB)
		{
			if(BB->hasName())
				errs()<<"\n   "<<BB->getName()<<":";


			//Iterate over Block BB's Instructions
			for (BasicBlock::iterator I = BB->begin(), E = BB->end(); I != E; ++I)
			{
				//errs()<<"\n\t"<<CiaoLiteral::toLiteral(I)<< "   END";
			}
			errs()<<"\n";
//			for (BasicBlock::iterator I = BB->begin(), E = BB->end(); I != E; ++I)
//			{
//				//Instruction* II = (Instruction*) I;
//
//
//				if(I->hasName())    // If an instruction outputs result
//				{
//					//Type and name of instruction value(what instruction returns)
//					errs()<<"\n     "<<*(I->getType() )<<" "<<I->getName();
//
//					if(I->isUsedOutsideOfBlock(BB))
//							errs()<<"-{O}";//<<"{used outside this block}";
//
//					errs()<<"  =  "<<I->getOpcodeName(); //getOpcode() returns a member of one of the enums like Instruction::Add.
////					if(I->isUsedOutsideOfBlock(BB))
////						errs()<<"{used outside this block}";
//
////					for (Value::use_iterator i = I->use_begin(), e = I->use_end(); i != e; ++i)
////						if (Instruction *Inst = dyn_cast<Instruction>(*i))
////						{
////							errs() << "( ==> Inst is used in <Block, Instruction>: ";
////							errs() << " < "<< Inst->getParent()->getName()<<", "<<*Inst << ")  ";
////						}
//				}
//				else if(true)        // Otherwise (no output)
//					errs()<<"\n     VOID = "<<I->getOpcodeName();
//				errs()<<"(";
//
//				//Instruction operands
//				for(unsigned int i=0;i<I->getNumOperands();i++)
//				{
//					errs()<<*(I->getOperand(i) ->getType())<<" ";
//
//					if(!(I->getOperand(i)->hasName()))
//					{ //Cases when operand does not have a name:
//					  // 1) Constant operand
//						if (ConstantInt *CI = dyn_cast<ConstantInt>(I->getOperand(i)))
//							if (CI->getBitWidth()<=64) //Checking for integer bits
//								errs() << CI->getSExtValue()<<",";
//					}
//					else
//						errs()<<I->getOperand(i)->getName()<<",";
//				}
//				errs()<<")";
//			}

		}
		errs()<<"\n";
		errs()<<"============END=================\n";

		return false;
	}

	virtual bool runOnModule(Module& M)
	{
//		errs()<<"before loopinfo in runOnModule \n";
//		//LI = &getAnalysis<LoopInfo>( &(M.getFunction("dloop")));
//		errs()<<"after loopinfo in runOnModule \n";


		CiaoModule * m = new CiaoModule(&M, this);

		//std::string ErrorInfo;
		std::error_code ErrorInfo;

		raw_fd_ostream O(OutputFilename.c_str(), ErrorInfo, sys::fs::F_None);

//		  std::error_code EC;
//		  raw_fd_ostream O(OutputFilename.c_str(), EC, sys::fs::F_None);

		//Transformation and output in O.
		m->print(O);

		//To print the transformed program on console
		//m->print(errs());
		delete m;

		return false;
	}

	virtual bool runOnModule_old(Module& M)
	{
//		std::cerr << "Hello\n"; //remove this
		for (Module::iterator MI = M.begin(), ME = M.end(); MI != ME; ++MI)
		{
			//runOnFunction(*MI);
		}
		return false;
	}


//	void printAssertions(raw_fd_ostream * OS)
//	{
//		std::ifstream ifs;
//
//		  ifs.open (AssertionFilename.c_str(), std::ifstream::in);
//
//		  char c = ifs.get();
//
//		  while (ifs.good()) {
//		    OS << c;
//		    c = ifs.get();
//		  }
//
//		  ifs.close();
//
//	}

	virtual void print(raw_ostream &O,const Module*) const {
	  //O << ".......Pass::print not implemented for pass: '" << getPassName() << "'!\n";
	}

	void print(string F){}

};

char llvm_to_ciao::ID = 0;
RegisterPass<llvm_to_ciao> X("emit-ciao","Ciao Transformation");


}




