:- module(xs1_asm_operand_util,
	[ xs1_op2ciao_op/2

	],
	[ assertions,
	  basicmodes
	]).

:- doc(title, "A parser and transformer for xs1-assembly operands").
:- doc(author,"Umer Liqat").

%:- doc(module, "This module implements predicates for atom 
%	to term conversion. Modified definition of atom2term").

:- doc(bug, "This module is a modified version of atom2term").

:- doc(bug,"This is just a quick hack written mainly for parsing and transforming
   xs1-assembly operands").

:- pred xs1_op2ciao_op(+Atom,-Term) # "Convert a xs1 operand into a ciao operand.
   @var{Atom} is an atom, but must have xs1-aseembly operand syntax.  @var{Term} is a
   term resulting operand syntax for ciao representation by parsing @var{Atom} char by char.".

xs1_op2ciao_op(Atom,Term):-
	atom_codes(Atom,String),
	parse_term(String,Term1, _, vars), 
	atom_codes(Term,Term1),!.

% Takes an atom of the form 'sp[0x1] and return sp and 0x1.
:- export(separate_atom/3).
separate_atom(Atom,Term11,Term22):-
	atom_codes(Atom,String),
	separate_atom_aux(String, Term1, Term2),
	atom_codes(Term11,Term1),
	atom_codes(Term22,Term2),!.

%separate_atom_aux([],[],[]).
separate_atom_aux([91|Rest],[], Term2):-
	separate_atom_aux(Rest, Term2, []), !.
separate_atom_aux([93],[], []).
separate_atom_aux([32|Rest],Term, Term2):-
	separate_atom_aux(Rest, Term, Term2),!.
separate_atom_aux([C|Rest],[C|Term], Term2):-
	separate_atom_aux(Rest, Term, Term2).

% :- pred parse_term(+String, -Term, ?Rest)
%       # "@var{String} is parsed into @var{Term} up to @var{Rest}
%          (which is the non-parsed rest of the list). The term is
%          assumed to be ground.".

% :- test parse_term(A,T,R) : ( A = "f(a)" ) => ( T = f(a), R = "" ).
% :- test parse_term(A,T,R) : ( A = "f(a) foo " ) => ( T = f(a), R = " foo " ).

% parse_term(String, Term, Rest) :- 
% 	parse_term(String, Term, Rest, novars).

:- pred parse_term(+String, -Term, ?Rest, +Vars)
      # "@var{String} is parsed into @var{Term} up to @var{Rest}
         (which is the non-parsed rest of the list). The term is
         assumed to be ground. If @var{Vars} is @tt{vars} then upper
         case identifiers in the term are turned into variables.".

parse_term([],'',[], _).
parse_term([C|String0],Term,String,Vars):-
	parse_term0(C,String0,Term,String,Vars).
	
parse_term0(0'\s,String0,Term,String,Vars):- !, % space
        parse_term(String0,Term,String,Vars).
parse_term0(0'\t,String0,Term,String,Vars):- !, % tab
        parse_term(String0,Term,String,Vars).
parse_term0(0'\n,String0,Term,String,Vars):- !, % newline
        parse_term(String0,Term,String,Vars).
parse_term0(0'[,String0,Term,String,Vars):- !, % list
	parse_args0(String0,Term,[0']|String],Vars).
parse_term0(0'",String0,Term,String,_):- !, % string
	parse_string(String0,Str,String),
	atom_codes(Term,Str).
parse_term0(0'( , _, _, _, _):- !, fail.
parse_term0(0') , _, _, _, _):- !, fail.
parse_term0(0'] , _, _, _, _):- !, fail.
parse_term0(0', , _, _, _, _):- !, fail.
parse_term0(C0,String0,Term,String,Vars):- % struct
        parse_functor1(C0,String0,FunctorStr,String1),
	(  FunctorStr = [U|_], Vars = vars, is_upper_case(U)
          ->
	   Term = FunctorStr,
           String = String1
	; false
	  ).

is_upper_case(Ch) :-
	Ch >= 65, Ch =< 90.

parse_functor(0'[, String,Functor,String1):-
        parse_functor_(String,Functor,String1).
parse_functor(0'], String,Functor,String1):-
        parse_functor_(String,Functor,String1).
parse_functor(0'-, String,[0'_|Functor],String1):-
        parse_functor_(String,Functor,String1).
parse_functor(C, String,[C|Functor],String1):-
        parse_functor_(String,Functor,String1).

parse_functor1(C, String,[D|Functor],String1):-
	capatilize(C,D),
        parse_functor_(String,Functor,String1).

capatilize(C,D):-
	D is C-32.
	
parse_functor_([], [], []).
parse_functor_([C|String],Functor,String1):-
	parse_functor(C,String,Functor,String1).

parse_args0(String0,[Arg|Args],String,Vars):-
	parse_term(String0,Arg,String1,Vars), !,
	parse_args1(String1,Args,String,Vars).
parse_args0(String, [], String,_).

parse_args1([0'\s|String0],Args,String,Vars) :- !,
        parse_args1(String0,Args,String,Vars).
parse_args1([0', |String0],[Arg|Args],String,Vars):- !,
	parse_term(String0,Arg,String1,Vars),
	parse_args1(String1,Args,String,Vars).
parse_args1(String,[],String,_).

parse_string([],[],[]).
parse_string([C|String],List,String1):-
	parse_string0(C,String,List,String1).

parse_string0(0'",String,[],String):- !.
parse_string0(C,String,[C|List],String1):-
	parse_string(String,List,String1).

