:- module(xs1_interpreter,[ interpret/5,inst_interpret/7, instr_op/5],[ assertions, regtypes]).

:- use_module(library(sets)).
:- use_module(library(messages)).
:- use_module(xs1_transformation,[cisC/4,cisC_prev/4,block_output_vars/2,block_input_vars/2,get_entry_input_vars/3% , def/2, ref/2
	]).
:- use_module(library(lists)).
:- use_module(xs1_asm_operand_util).

:- doc(title, "XC Assembly Interpreter").
:- doc(author, "Umer Liqat").

:- doc(summary, "A Interpreter for XS1 assembly instruction set. The
   interpreter is used in graph construction and SSA form
   construction.").

:- doc(module, "This module implements a naive interpreter to
   interpret each instruction and extract the information of how
   operands of each instructions are modified. The resultant ref/def
   sets are stored in dynamic predicate @var{ref/2} @var{def/2}.  ").

:- doc(bug, "interpret uses dynamic predicates, remove dynamic predicates to return the ref/def as output args @var{inst_format}").
:- doc(bug, "Produce correct Literal from interpret to print in output file").

%%%%%%%%%%%%%%%%%%%% Interpreter %%%%%%%%%%%%%%

%removeIndirection(_,_).
% removeIndirection([O1,O2],[O11,O22]):-
% 	O=sp[X],
%	atom_concat(sp,X,Oout).
% :- export(def_ref_sets/3).
% def_ref_sets(Node,DefSet,RefSet):-
% 	def(DefSet),
% 	functor(DefSet,Node,_),
% 	ref(RefSet),
% 	functor(RefSet,Node,_).
%flattenOperand(O,Oout):-
%	O=sp[X],
%	atom_concat(sp,X,Oout).
%:- export(interpret/4).
%% interpret(Name, Node,Instruction,Format, Operands)
%% Literal is the equivalent effect of instrunction for the abstract prolog program

:- doc(inst_interpret/7, "It recieves an instruction tuple and returns
   different elements from the tuple as well as the def and ref sets
   based on the semantics of the instruction..
   @Includedef{inst_interpret/7}").

%inst_interpret(Instruction, Address, Operands, DefSet, RefSet, InstructionName, InstructionType).

inst_interpret(instr(_L,_A,entsp,u6,operands(O1)), _A,[O1], [], [], entsp, u6).

inst_interpret(instr(_L,_A,stw,ru6,operands(O1,O2)), _A,[O1,O2], [O2], O11, stw, ru6):-
	(operandType(O1,num)->
	 O11=[];
	    O11=[O1]
	).

inst_interpret(instr(_L, _A, stw, l3r,operands(O1,O2,O3)), _A,[O1,O2,O3], [O1], [O2, O3], stw, l3r).

inst_interpret(instr(_L,_A,stw,lru6,operands(O1,O2)), _A,[O1,O2],[O2],[O11],stw,lru6):-
	(operandType(O1,num)->
	 O11=[];
	    O11=[O1]
	).

% inst_interpret(instr(_L,_A,stw,'2rus',operands(O1,O2)), _A,[O1,O2], [O2],O11,stw,'2rus'):-
% 	(operandType(O1,num)->
% 	 O11=[];
% 	    O11=[O1]
% 	).

inst_interpret(instr(_L,_A,stw,'2rus',operands(O1,O2,O3)), _A,[O1,O2,O3],[O2],[O1],stw,'2rus').

inst_interpret(instr(_L,_A,ldw,ru6,operands(O1,O2)), _A,[O1,O2],[O1], O22, ldw, ru6 ):-
	(operandType(O2,num)->
	 O22=[];
	    O22=[O2]
	).

inst_interpret(instr(_L,_A,ldw,'2rus',operands(O1,O2,O3)), _A,[O1,O2,O3],[O1],[O2],ldw,'2rus').

inst_interpret(instr(_L,_A,ldw, lru6, operands(O1,O2)), _A,[O1,O2],O1,O22,ldw,lru6):-
	(operandType(O2,num)->
	 O22=[];
	    O22=[O2]
	).

inst_interpret(instr(_L,_A,ldw,'3r',operands(O1,O2,O3)), _A,[O1,O2,O3],[O1],[O2,O3],ldw, '3r' ).

inst_interpret(instr(_L,_A,ldaw,'3r',operands(O1,O2,O3)), _A,[O1,O2,O3],[O1],[O2,O3],ldaw, '3r' ).

inst_interpret(instr(_L,_A,ld8u,'3r',operands(O1,O2,O3)), _A,[O1,O2,O3],[O1],[O2,O3],ld8u, '3r' ).

inst_interpret(instr(_L,_A,ldaw, ru6, operands(O1,O2)), _A,[O1,O2],[O1],O22,ldaw,ru6):-
	(operandType(O2,num)->
	 O22=[];
	    O22=[O2]
	).
	
inst_interpret(instr(_L,_A,ldaw, lru6, operands(O1,O2)), _A,[O1,O2],[O1],O22,ldaw, lru6):-
	(operandType(O2,num)->
	 O22=[];
	    O22=[O2]
	).

inst_interpret(instr(_L,_A,ldaw,'2rus',operands(O1,O2)), _A,[O1,O2],[O1],O22,ldaw, '2rus' ):-
	(operandType(O2,num)->
	 O22=[];
	    O22=[O2]
	).

inst_interpret(instr(_L,_A,ldaw,'3r',operands(O1,O2,O3)), _A,[O1,O2,O3],[O1],[O2,O3],ldaw, '3r' ).

inst_interpret(instr(_L,_A,ldc,lru6,operands(O1,O2)), _A,[O1,O2],[O1],[],ldc,lru6).

inst_interpret(instr(_L,_A,ldc,ru6,operands(O1,O2)), _A,[O1,O2],[O1],[],ldc,ru6).

inst_interpret(instr(_L,_A,bf,ru6,operands(O1,_O2)), _A,[O1,'label'],[],[O1],bf, ru6).

inst_interpret(instr(_L,_A,bt,ru6,operands(O1,_O2)), _A,[O1,'label'],[],[O1],bt, ru6).

inst_interpret(instr(_L,_A,bu,u6,operands(_O1)), _A,['label'],[],[],bu, u6).

inst_interpret(instr(_L,_A,mkmsk,rus,operands(O1,O2)), _A,[O1,O2],[O1],[],mkmsk, rus).

inst_interpret(instr(_L,_A,retsp,u6,operands(O1)), _A,[O1],[],[],retsp,u6).

inst_interpret(instr(_L,_A,bl,u10,operands(O1)), _A,[O1|Operands], Output, Input, bl, u10):-
	get_entry_input_vars(O1, Input, Output),
	append(Input, Output, Operands).

inst_interpret(instr(_L,_A,lss,'3r',operands(O1,O2,O3)), _A,[O1,O2,O3], [O1], [O2,O3], lss, '3r').% :-
	% genRefSet([O2,O3],O).

inst_interpret(instr(_L,_A,eq,'3r',operands(O1,O2,O3)), _A,[O1,O2,O3], [O1], [O2, O3], eq, '3r' ).

inst_interpret(instr(_L,_A,sub,'3r',operands(O1,O2,O3)), _A,[O1,O2,O3], [O1], [O2, O3], sub, '3r' ).

inst_interpret(instr(_L,_A,add,'3r',operands(O1,O2,O3)), _A,[O1,O2,O3], [O1], [O2, O3], add, '3r' ).

inst_interpret(instr(_L,_A,mul,'3r',operands(O1,O2,O3)), _A,[O1,O2,O3], [O1], [O2, O3], mul, '3r' ).

inst_interpret(instr(_L,_A,or,'3r',operands(O1,O2,O3)), _A,[O1,O2,O3], [O1], [O2, O3], or, '3r' ).

inst_interpret(instr(_L,_A,mul,l3r,operands(O1,O2,O3)), _A,[O1,O2,O3], [O1], [O2, O3], mul, l3r ).

inst_interpret(instr(_L,_A,sub,'2rus',operands(O1,O2,O3)), _A,[O1,O2,O3], [O1], [O2], sub, '2rus' ).

inst_interpret(instr(_L,_A,add,'2rus',operands(O1,O2,O3)), _A,[O1,O2,O3], [O1], [O2], add, '2rus' ).

inst_interpret(instr(_L,_A,eq,'2rus',operands(O1,O2,O3)), _A,[O1,O2,O3], [O1], [O2], eq, '2rus' ).

inst_interpret(instr(_L,_A,shl,'2rus',operands(O1,O2,O3)), _A,[O1,O2,O3], [O1], [O2], shl, '2rus' ).

inst_interpret(instr(_L,_A,rems,l3r,operands(O1,O2,O3)), _A,[O1,O2,O3], [O1], [O2, O3], rems, l3r ).

inst_interpret(instr(_L,_A,divs,l3r,operands(O1,O2,O3)), _A,[O1,O2,O3], [O1], [O2, O3], divs, l3r ).

inst_interpret(instr(_L,_A,ashr,l2rus,operands(O1,O2,O3)), _A,[O1,O2,O3], [O1], [O2], ashr, l2rus ).

inst_interpret(instr(_L,_A,shr,'3r',operands(O1,O2,O3)), _A,[O1,O2,O3], [O1], [O2, O3], shr, '3r' ).

inst_interpret(instr(_L,_A,zext,rus,operands(O1,O3)), _A,[O1,O2,O3],[O1],[O2],zext, rus ):-
	O2=O1.

inst_interpret(instr(_L,_A,maccs,l4r,operands(D,E,X,Y)), _A,[D,E,D,E,X,Y], [D,E], [D,E,X,Y], maccs, l4r ).

inst_interpret(instr(_L,_A,sext,'2r',operands(D,S)), _A,[D,D,S], [D], [D,S], maccs, '2r' ).

inst_interpret(instr(_L,_A,sext,rus,operands(D,S)), _A,[D,D,S], [D], [D], maccs, rus ).

% inst_interpret(instr(_,_,I,T,_),_,_,_,_,I,T):-
% 	error_message("Instruction not recognized ~q, ~q",[I, T]),nl,nl,nl, fail.
	
% inst_interpret(calll(_X, _CallCounter),nil,[],[],[],nil,nil).
inst_interpret(jmp(_X),nil,[],[],[],nil,nil).
inst_interpret(branch(_X, _CallCounter),nil,[],[],[],nil,nil).

:- doc(interpret/5, "It receives the Name of the block, Node in the
program CFG, Instruction tuple on that Node and a previous node
PrevNode. It returns an abstracted ciao literal in Literal.
@Includedef{inst_interpret/7}").

% interpret(Name, Node, calll(X, _CallCounter), Literal,PrevNode):-!,
% 	(block_output_vars(X, OutputVars),
% 	 block_input_vars(X, InputVars),
% 	 getNewOutputVars(Name, Node, OutputVars, NewOutputVars),
% 	 getNewInputVars(Name, PrevNode, InputVars, NewInputVars),
% 	 append(NewInputVars, NewOutputVars, Arguments)
% 	->
% 	Literal=..[X|Arguments];
% 	    Literal=calll(X, _CallCounter)
% 	).

interpret(_Name, _Node,branch(X,Y),[branch(X,Y)],_):-!.

interpret(Name, Node, jmp(X), [Literal],PrevNode):-!,
	(block_output_vars(X, OutputVars),
	 block_input_vars(X, InputVars),
	 getNewOutputVars(Name, Node, OutputVars, NewOutputVars),
	 getNewInputVars(Name, PrevNode, InputVars, NewInputVars),
	 append(NewInputVars, NewOutputVars, Arguments)
	->
	Literal=..[X|Arguments];
	    Literal=jmp(X)
	).

interpret(Name, Node, Instruction, [L1,L2], PrevNode):-
	inst_interpret(Instruction, _A, [Func|Operands], Def, Ref, bl, Type),!,
	cisC_list(Name, Node, Def, Def_new),
	cisC_prev_list(Name, PrevNode, Ref, Ref_new),
	get_ssa_ops(Operands, (Ref, Ref_new), (Def, Def_new), SSA_operands),
	atom_concat(bl, '_', Inst_type1),
	atom_concat(Inst_type1, Type, Inst_type),
	L1=..[Inst_type,Func],
	L2=..[Func|SSA_operands].

interpret(Name, Node, Instruction, [Literal], PrevNode):-
	inst_interpret(Instruction, _A, Operands, Def, Ref, Inst, Type),
	cisC_list(Name, Node, Def, Def_new),
	cisC_prev_list(Name, PrevNode, Ref, Ref_new),
	get_ssa_ops(Operands, (Def, Def_new), (Ref, Ref_new), SSA_operands),
	atom_concat(Inst, '_', Inst_type1),
	atom_concat(Inst_type1, Type, Inst_type),	
	append([Inst_type], SSA_operands, Lit),
	Literal=..Lit.

cisC_prev_list(_Name, _Node, [], []).
cisC_prev_list(Name, Node, [R|Rs], [Rn|Rns]):-
	cisC_prev(Name, Node, R, Rn),
	cisC_prev_list(Name, Node, Rs, Rns).

cisC_list(_Name, _Node, [], []).
cisC_list(Name, Node, [D|Ds], [Dn|Dns]):-
	cisC(Name, Node, D, Dn),
	cisC_list(Name, Node, Ds, Dns).
	
get_ssa_ops([], _Def, _Ref,  []).
get_ssa_ops([O|Os], Def, Ref,  [O_new|Operands]):-
	Def=(Def1, _Def_new),
	member(O, Def1),!,
	get_new(O, Def, O_new, Def_modified),
	get_ssa_ops(Os, Def_modified, Ref,  Operands).
get_ssa_ops([O|Os], Def, Ref,  [O_new|Operands]):-
	Ref=(Ref1, _Ref_new),
	member(O, Ref1),!,
	get_new(O, Ref, O_new, Ref_modified),
	get_ssa_ops(Os, Def, Ref_modified,  Operands).

get_ssa_ops([O|Os], Def, Ref,  [O|Operands]):-
	%atom_concat(O,'',OO),
	get_ssa_ops(Os, Def, Ref,  Operands).

get_new(O, ([O|_Ds], [Dn|_Dns]), Dn, (_Ds,_Dns) ).
get_new(O, ([_D|Ds], [_Dn|Dns]), Dn,  ([_D|Ds_new], [_Dn|Dns_new]) ):-
	get_new(O,(Ds,Dns), Dn, (Ds_new,Dns_new) ).
	
% interpret_bak(Name, _Node,instr(_L,A,entsp,u6,operands(O1)), Literal, _Ref, _Def, _PrevNode):-
% 	Literal=..[entsp_u6,O1],
% 	data_facts:asserta_fact(def(Name-A,[])),
% 	data_facts:asserta_fact(ref(Name-A,[])).

% interpret_bak(Name, Node,instr(_L,A,stw,ru6,operands(O1,O2)), Literal, _Ref, _Def, PrevNode):-
% 	(cisC(Name,Node,O2,O2_)->
% 	 O22=O2_;
% 	    O22=O2
% 	),
% 	data_facts:asserta_fact(def(Name-A,[O2])),
% 	(operandType(O1,num)->
% 	 data_facts:asserta_fact(ref(Name-A,[]));
% 	    (cisC_prev(Name,PrevNode,O1,O1_)->
% 	     O11=O1_;
% 		O11=O1
% 	    ),
% 	    data_facts:asserta_fact(ref(Name-A,[O1]))
% 	),
% 	Literal=..[stw_ru6,O11,O22].

% interpret_bak(Name, Node,instr(_L,A,stw,'l3r',operands(O1,O2,O3)), Literal, _Ref, _Def, PrevNode):-
% 	(cisC_prev(Name,Node,O1,O1_)->
% 	 O11=O1_;
% 	    O11=O1
% 	),

% 	(cisC_prev(Name,PrevNode,O2,O2_)->
% 	 O22=O2_;
% 	    O22=O2
% 	),

% 	(cisC_prev(Name,PrevNode,O3,O3_)->
% 	 O33=O3_;
% 	    O33=O3
% 	),
% 	%genRefSet([O2,O3],O),
% 	data_facts:asserta_fact(def(Name-A,[O1])),
% 	data_facts:asserta_fact(ref(Name-A,[O2, O3])),
% 	Literal=..[stw_l3r,O11,O22,O33].

% interpret_bak(Name, Node,instr(_L,A,stw,lru6,operands(O1,O2)), Literal, _Ref, _Def, PrevNode):-
% 	(cisC(Name,Node,O2,O2_)->
% 	 O22=O2_;
% 	    O22=O2
% 	),
% 	data_facts:asserta_fact(def(Name-A,[O2])),
% 	(operandType(O1,num)->
% 	 data_facts:asserta_fact(ref(Name-A,[]));
% 	    (cisC_prev(Name,PrevNode,O1,O1_)->
% 	     O11=O1_;
% 		O11=O1
% 	    ),
% 	    data_facts:asserta_fact(ref(Name-A,[O1]))
% 	),
% 	Literal=..[stw_lru6,O11,O22].

% interpret_bak(Name, Node,instr(_L,A,stw,'2rus',operands(O1,O2)), Literal, _Ref, _Def, PrevNode):-
% 	(cisC(Name,Node,O2,O2_)->
% 	 O22=O2_;
% 	    O22=O2
% 	),
% 	data_facts:asserta_fact(def(Name-A,[O2])),
% 	(operandType(O1,num)->
% 	 data_facts:asserta_fact(ref(Name-A,[]));
% 	    (cisC_prev(Name,PrevNode,O1,O1_)->
% 	     O11=O1_;
% 		O11=O1
% 	    ),
% 	    data_facts:asserta_fact(ref(Name-A,[O1]))
% 	),
% 	Literal=..[stw_2rus,O11,O22].

% interpret_bak(Name, Node,instr(_L,A,ldw,ru6,operands(O1,O2)), Literal, _Ref, _Def, PrevNode):-
% 	% (cisC(Name,Node,O1,O1_)->
% % 	 Literal=..[ldw,O1_,O2];
% % 	    Literal=..[ldw,O1,O2]
% % 	),
% 	(cisC(Name,Node,O1,O1_)->
% 	 O11=O1_;
% 	    O11=O1
% 	),
% 	data_facts:asserta_fact(def(Name-A,[O1])),
% 	(operandType(O2,num)->
% 	 data_facts:asserta_fact(ref(Name-A,[]));
% 	    (cisC_prev(Name,PrevNode,O2,O2_)->
% 	     O22=O2_;
% 		O22=O2
% 	     ),
% 	    data_facts:asserta_fact(ref(Name-A,[O2]))
% 	),
% 	Literal=..[ldw_ru6,O11,O22].

% interpret_bak(Name, Node,instr(_L,A,ldw,'2rus',operands(O1,O2)), Literal, _Ref, _Def, PrevNode):-
% 	% (cisC(Name,Node,O1,O1_)->
% % 	 Literal=..[ldw,O1_,O2];
% % 	    Literal=..[ldw,O1,O2]
% % 	),
% 	(cisC(Name,Node,O1,O1_)->
% 	 O11=O1_;
% 	    O11=O1
% 	),
% 	data_facts:asserta_fact(def(Name-A,[O1])),
% 	(operandType(O2,num)->
% 	 data_facts:asserta_fact(ref(Name-A,[]));
% 	    (cisC_prev(Name,PrevNode,O2,O2_)->
% 	     O22=O2_;
% 		O22=O2
% 	     ),
% 	    data_facts:asserta_fact(ref(Name-A,[O2]))
% 	),
% 	Literal=..[ldw_2rus,O11,O22].

% interpret_bak(Name, Node,instr(_L,A,ldw, lru6, operands(O1,O2)), Literal, _Ref, _Def, PrevNode):-
% 	% (cisC(Name,Node,O1,O1_)->
% % 	 Literal=..[ldw,O1_,O2];
% % 	    Literal=..[ldw,O1,O2]
% % 	),
% 	(cisC(Name,Node,O1,O1_)->
% 	 O11=O1_;
% 	    O11=O1
% 	),
% 	data_facts:asserta_fact(def(Name-A,[O1])),
% 	(operandType(O2,num)->
% 	 data_facts:asserta_fact(ref(Name-A,[]));
% 	    (cisC_prev(Name,PrevNode,O2,O2_)->
% 	     O22=O2_;
% 		O22=O2
% 	     ),
% 	    data_facts:asserta_fact(ref(Name-A,[O2]))
% 	),
% 	Literal=..[ldw_lru6,O11,O22].

% interpret_bak(Name, Node,instr(_L,A,ldw,'3r',operands(O1,O2,O3)), Literal, _Ref, _Def, PrevNode):-
% 	(cisC(Name,Node,O1,O1_)->
% 	 O11=O1_;
% 	    O11=O1
% 	),

% 	(cisC_prev(Name,PrevNode,O2,O2_)->
% 	 O22=O2_;
% 	    O22=O2
% 	),

% 	(cisC_prev(Name,PrevNode,O3,O3_)->
% 	 O33=O3_;
% 	    O33=O3
% 	),
% 	%genRefSet([O2,O3],O),
% 	data_facts:asserta_fact(def(Name-A,[O1])),
% 	data_facts:asserta_fact(ref(Name-A,[O2, O3])),
% 	Literal=..[ldw_3r,O11,O22,O33].


% interpret_bak(Name, Node,instr(_L,A,ldaw, ru6, operands(O1,O2)), Literal, _Ref, _Def, PrevNode):-
% 	% (cisC(Name,Node,O1,O1_)->
% % 	 Literal=..[ldw,O1_,O2];
% % 	    Literal=..[ldw,O1,O2]
% % 	),
% 	(cisC(Name,Node,O1,O1_)->
% 	 O11=O1_;
% 	    O11=O1
% 	),
% 	data_facts:asserta_fact(def(Name-A,[O1])),
% 	(operandType(O2,num)->
% 	 data_facts:asserta_fact(ref(Name-A,[]));
% 	    (cisC_prev(Name,PrevNode,O2,O2_)->
% 	     O22=O2_;
% 		O22=O2
% 	     ),
% 	    data_facts:asserta_fact(ref(Name-A,[O2]))
% 	),
% 	Literal=..[ldaw_ru6,O11,O22].

% interpret_bak(Name, Node,instr(_L,A,ldaw, lru6, operands(O1,O2)), Literal, _Ref, _Def, PrevNode):-
% 	% (cisC(Name,Node,O1,O1_)->
% % 	 Literal=..[ldw,O1_,O2];
% % 	    Literal=..[ldw,O1,O2]
% % 	),
% 	(cisC(Name,Node,O1,O1_)->
% 	 O11=O1_;
% 	    O11=O1
% 	),
% 	data_facts:asserta_fact(def(Name-A,[O1])),
% 	(operandType(O2,num)->
% 	 data_facts:asserta_fact(ref(Name-A,[]));
% 	    (cisC_prev(Name,PrevNode,O2,O2_)->
% 	     O22=O2_;
% 		O22=O2
% 	     ),
% 	    data_facts:asserta_fact(ref(Name-A,[O2]))
% 	),
% 	Literal=..[ldaw_lru6,O11,O22].

% interpret_bak(Name, Node,instr(_L,A,ldaw,'2rus',operands(O1,O2)), Literal, _Ref, _Def, PrevNode):-
% 	% (cisC(Name,Node,O1,O1_)->
% % 	 Literal=..[ldw,O1_,O2];
% % 	    Literal=..[ldw,O1,O2]
% % 	),
% 	(cisC(Name,Node,O1,O1_)->
% 	 O11=O1_;
% 	    O11=O1
% 	),
% 	data_facts:asserta_fact(def(Name-A,[O1])),
% 	(operandType(O2,num)->
% 	 data_facts:asserta_fact(ref(Name-A,[]));
% 	    (cisC_prev(Name,PrevNode,O2,O2_)->
% 	     O22=O2_;
% 		O22=O2
% 	     ),
% 	    data_facts:asserta_fact(ref(Name-A,[O2]))
% 	),
% 	Literal=..[ldaw_2rus,O11,O22].

% interpret_bak(Name, Node,instr(_L,A,ldaw,'3r',operands(O1,O2,O3)), Literal, _Ref, _Def, PrevNode):-
% 	(cisC(Name,Node,O1,O1_)->
% 	 O11=O1_;
% 	    O11=O1
% 	),

% 	(cisC_prev(Name,PrevNode,O2,O2_)->
% 	 O22=O2_;
% 	    O22=O2
% 	),

% 	(cisC_prev(Name,PrevNode,O3,O3_)->
% 	 O33=O3_;
% 	    O33=O3
% 	),
% 	%genRefSet([O2,O3],O),
% 	data_facts:asserta_fact(def(Name-A,[O1])),
% 	data_facts:asserta_fact(ref(Name-A,[O2, O3])),
% 	Literal=..[ldaw_3r,O11,O22,O33].

% interpret_bak(Name, Node,instr(_L,A,ldc,ru6,operands(O1,O2)), Literal, _Ref, _Def, _PrevNode):-
% 	(cisC(Name,Node,O1,O1_)->
% 	 Literal=..[ldc_ru6,O1_,O2];
% 	    Literal=..[ldc_ru6,O1,O2]
% 	),
% 	data_facts:asserta_fact(def(Name-A,[O1])),
% 	data_facts:asserta_fact(ref(Name-A,[])).

% interpret_bak(Name, _Node,instr(_L,A,bf,ru6,operands(O1,__O2)), Literal, _Ref, _Def, PrevNode):-
% 	data_facts:asserta_fact(def(Name-A,[])),
% 	(cisC_prev(Name,PrevNode,O1,O1_)->
% 	 O11=O1_;
% 	    O11=O1
% 	),
% 	Literal=..[bf_ru6,O11,_O2],
% 	data_facts:asserta_fact(ref(Name-A,[O1])).
% interpret_bak(Name, _Node,instr(_L,A,bt,ru6,operands(O1,__O2)), Literal, _Ref, _Def, PrevNode):-
% 	data_facts:asserta_fact(def(Name-A,[])),
% 	(cisC_prev(Name,PrevNode,O1,O1_)->
% 	 O11=O1_;
% 	    O11=O1
% 	),
% 	Literal=..[bt_ru6,O11,_O2],
% 	data_facts:asserta_fact(ref(Name-A,[O1])).

% interpret_bak(Name, _Node,instr(_L,A,bu,u6,operands(__O1)), Literal, _Ref, _Def, _PrevNode):-
% 	Literal=..[bu_u6,_O1],
% 	data_facts:asserta_fact(def(Name-A,[])),
% 	data_facts:asserta_fact(ref(Name-A,[])).

% interpret_bak(Name, Node,instr(_L,A,mkmsk,rus,operands(O1,O2)), Literal, _Ref, _Def, _PrevNode):-
% 	(cisC(Name,Node,O1,O1_)->
% 	 Literal=..[mkmsk_rus,O1_,O2];
% 	    Literal=..[mkmsk_rus,O1,O2]
% 	),
% 	data_facts:asserta_fact(def(Name-A,[O1])),
% 	data_facts:asserta_fact(ref(Name-A,[])).

% interpret_bak(Name, _Node,instr(_L,A,retsp,u6,operands(O1)), Literal, _Ref, _Def, _PrevNode):-
% 	Literal=..[retsp_u6,O1],
% 	data_facts:asserta_fact(def(Name-A,[])),
% 	data_facts:asserta_fact(ref(Name-A,[])).

% interpret_bak(Name, _Node,instr(_L,A,bl,u10,operands(__O1)), Literal, _Ref, _Def, _PrevNode):-
% 	Literal=..[bl_u10,_O1],
% 	data_facts:asserta_fact(def(Name-A,[])),
% 	data_facts:asserta_fact(ref(Name-A,[])).

% % interpret_bak(Name, Node,instr(_L,A,lss,'3r',operands(O1,O2,O3)), Literal, _Ref, _Def, PrevNode):-
% % 	(cisC(Name,Node,O1,O1_)->
% % 	 O11=O1_;
% % 	    O11=O1
% % 	),

% % 	(cisC_prev(Name,PrevNode,O2,O2_)->
% % 	 O22=O2_;
% % 	    O22=O2
% % 	),

% % 	(cisC_prev(Name,PrevNode,O3,O3_)->
% % 	 O33=O3_;
% % 	    O33=O3
% % 	),
% % 	genRefSet([O2,O3],O),
% % 	data_facts:asserta_fact(def(Name-A,[O1])),
% % 	data_facts:asserta_fact(ref(Name-A,O)),
% % 	Literal=..[lss_3r,O11,O22,O33].

% interpret_bak(Name, Node,instr(_L,A,lss,'3r',operands(O1,O2,O3)), Literal, _Ref, _Def, PrevNode):-
% 	(cisC(Name,Node,O1,O1_)->
% 	 O11=O1_;
% 	    O11=O1
% 	),

% 	(cisC_prev(Name,PrevNode,O2,O2_)->
% 	 O22=O2_;
% 	    O22=O2
% 	),

% 	(cisC_prev(Name,PrevNode,O3,O3_)->
% 	 O33=O3_;
% 	    O33=O3
% 	),
% 	%genRefSet([O2,O3],O),
% 	data_facts:asserta_fact(def(Name-A,[O1])),
% 	data_facts:asserta_fact(ref(Name-A,[O2, O3])),
% 	Literal=..[lss_3r,O11,O22,O33].


% interpret_bak(Name, Node,instr(_L,A,eq,'3r',operands(O1,O2,O3)), Literal, _Ref, _Def, PrevNode):-
% 	(cisC(Name,Node,O1,O1_)->
% 	 O11=O1_;
% 	    O11=O1
% 	),

% 	(cisC_prev(Name,PrevNode,O2,O2_)->
% 	 O22=O2_;
% 	    O22=O2
% 	),

% 	(cisC_prev(Name,PrevNode,O3,O3_)->
% 	 O33=O3_;
% 	    O33=O3
% 	),
% 	%genRefSet([O2,O3],O),
% 	data_facts:asserta_fact(def(Name-A,[O1])),
% 	data_facts:asserta_fact(ref(Name-A,[O2, O3])),
% 	Literal=..[eq_3r,O11,O22,O33].

% interpret_bak(Name, Node,instr(_L,A,sub,'3r',operands(O1,O2,O3)), Literal, _Ref, _Def, PrevNode):-
% 	(cisC(Name,Node,O1,O1_)->
% 	 O11=O1_;
% 	    O11=O1
% 	),

% 	(cisC_prev(Name,PrevNode,O2,O2_)->
% 	 O22=O2_;
% 	    O22=O2
% 	),

% 	(cisC_prev(Name,PrevNode,O3,O3_)->
% 	 O33=O3_;
% 	    O33=O3
% 	),
% 	%genRefSet([O2,O3],O),
% 	data_facts:asserta_fact(def(Name-A,[O1])),
% 	data_facts:asserta_fact(ref(Name-A,[O2, O3])),
% 	Literal=..[sub_3r,O11,O22,O33].

% interpret_bak(Name, Node,instr(_L,A,sub,'2rus',operands(O1,O2,O3)), Literal, _Ref, _Def, PrevNode):-
% 	(cisC(Name,Node,O1,O1_)->
% 	 O11=O1_;
% 	    O11=O1
% 	),

% 	(cisC_prev(Name,PrevNode,O2,O2_)->
% 	 O22=O2_;
% 	    O22=O2
% 	),

% 	(cisC_prev(Name,PrevNode,O3,O3_)->
% 	 O33=O3_;
% 	    O33=O3
% 	),
% %	genRefSet([O2,O3],O),
% 	data_facts:asserta_fact(def(Name-A,[O1])),
% 	data_facts:asserta_fact(ref(Name-A,[O2])),
% 	Literal=..[sub_2rus,O11,O22,O33].


% % 	(cisC(Name,Node,O1,O1_)->
% % 	 Literal=..[sub,O1_,O2,O3];
% % 	    Literal=..[sub,O1,O2,O3]
% % 	),
% % 	genRefSet([O2,O3],O),
% % 	data_facts:asserta_fact(def(A,[O1])),
% % 	data_facts:asserta_fact(ref(A,O)).

% interpret_bak(Name, Node,instr(_L,A,mul,'l3r',operands(O1,O2,O3)), Literal, _Ref, _Def, PrevNode):-
% 	(cisC(Name,Node,O1,O1_)->
% 	 O11=O1_;
% 	    O11=O1
% 	),

% 	(cisC_prev(Name,PrevNode,O2,O2_)->
% 	 O22=O2_;
% 	    O22=O2
% 	),

% 	(cisC_prev(Name,PrevNode,O3,O3_)->
% 	 O33=O3_;
% 	    O33=O3
% 	),
% 	%genRefSet([O2,O3],O),
% 	data_facts:asserta_fact(def(Name-A,[O1])),
% 	data_facts:asserta_fact(ref(Name-A,[O2, O3])),
% 	Literal=..[mul_l3r,O11,O22,O33].

% interpret_bak(Name, Node,instr(_L,A,rems,'l3r',operands(O1,O2,O3)), Literal, _Ref, _Def, PrevNode):-
% 	(cisC(Name,Node,O1,O1_)->
% 	 O11=O1_;
% 	    O11=O1
% 	),

% 	(cisC_prev(Name,PrevNode,O2,O2_)->
% 	 O22=O2_;
% 	    O22=O2
% 	),

% 	(cisC_prev(Name,PrevNode,O3,O3_)->
% 	 O33=O3_;
% 	    O33=O3
% 	),
% 	%genRefSet([O2,O3],O),
% 	data_facts:asserta_fact(def(Name-A,[O1])),
% 	data_facts:asserta_fact(ref(Name-A,[O2, O3])),
% 	Literal=..[rems_l3r,O11,O22,O33].


% interpret_bak(Name, Node,instr(_L,A,divs,'l3r',operands(O1,O2,O3)), Literal, _Ref, _Def, PrevNode):-
% 	(cisC(Name,Node,O1,O1_)->
% 	 O11=O1_;
% 	    O11=O1
% 	),

% 	(cisC_prev(Name,PrevNode,O2,O2_)->
% 	 O22=O2_;
% 	    O22=O2
% 	),

% 	(cisC_prev(Name,PrevNode,O3,O3_)->
% 	 O33=O3_;
% 	    O33=O3
% 	),
% 	%genRefSet([O2,O3],O),
% 	data_facts:asserta_fact(def(Name-A,[O1])),
% 	data_facts:asserta_fact(ref(Name-A,[O2, O3])),
% 	Literal=..[divs_l3r,O11,O22,O33].

% interpret_bak(Name, Node,instr(_L,A,add,'2rus',operands(O1,O2,O3)), Literal, _Ref, _Def, PrevNode):-
% 	(cisC(Name,Node,O1,O1_)->
% 	 O11=O1_;
% 	    O11=O1
% 	),

% 	(cisC_prev(Name,PrevNode,O2,O2_)->
% 	 O22=O2_;
% 	    O22=O2
% 	),

% 	(cisC_prev(Name,PrevNode,O3,O3_)->
% 	 O33=O3_;
% 	    O33=O3
% 	),
% 	%genRefSet([O2,O3],O),
% 	data_facts:asserta_fact(def(Name-A,[O1])),
% 	data_facts:asserta_fact(ref(Name-A,[O2])),
% 	Literal=..[add_2rus,O11,O22,O33].

% interpret_bak(Name, Node,instr(_L,A,eq,'2rus',operands(O1,O2,O3)), Literal, _Ref, _Def, PrevNode):-
% 	(cisC(Name,Node,O1,O1_)->
% 	 O11=O1_;
% 	    O11=O1
% 	),

% 	(cisC_prev(Name,PrevNode,O2,O2_)->
% 	 O22=O2_;
% 	    O22=O2
% 	),

% 	(cisC_prev(Name,PrevNode,O3,O3_)->
% 	 O33=O3_;
% 	    O33=O3
% 	),
% %	genRefSet([O2,O3],O),
% 	data_facts:asserta_fact(def(Name-A,[O1])),
% 	data_facts:asserta_fact(ref(Name-A,[O2])),
% 	Literal=..[eq_2rus,O11,O22,O33].

% interpret_bak(Name, Node,instr(_L,A,ashr,'l2rus',operands(O1,O2,O3)), Literal, _Ref, _Def, PrevNode):-
% 	(cisC(Name,Node,O1,O1_)->
% 	 O11=O1_;
% 	    O11=O1
% 	),

% 	(cisC_prev(Name,PrevNode,O2,O2_)->
% 	 O22=O2_;
% 	    O22=O2
% 	),

% 	(cisC_prev(Name,PrevNode,O3,O3_)->
% 	 O33=O3_;
% 	    O33=O3
% 	),
% 	%genRefSet([O2,O3],O),
% 	data_facts:asserta_fact(def(Name-A,[O1])),
% 	data_facts:asserta_fact(ref(Name-A,[O2])),
% 	Literal=..[ashr_l2rus,O11,O22,O33].

% interpret_bak(Name, Node,instr(_L,A,shr,'3r',operands(O1,O2,O3)), Literal, _Ref, _Def, PrevNode):-
% 	(cisC(Name,Node,O1,O1_)->
% 	 O11=O1_;
% 	    O11=O1
% 	),

% 	(cisC_prev(Name,PrevNode,O2,O2_)->
% 	 O22=O2_;
% 	    O22=O2
% 	),

% 	(cisC_prev(Name,PrevNode,O3,O3_)->
% 	 O33=O3_;
% 	    O33=O3
% 	),
% 	%genRefSet([O2,O3],O),
% 	data_facts:asserta_fact(def(Name-A,[O1])),
% 	data_facts:asserta_fact(ref(Name-A,[O2])),% O3 should be in ref too
% 	Literal=..[ashr_3r,O11,O22,O33].

% interpret_bak(Name, Node,instr(_L,A,zext,'rus',operands(O1,O3)), Literal, _Ref, _Def, PrevNode):-
% 	O2=O1,
% 	(cisC(Name,Node,O1,O1_)->
% 	 O11=O1_;
% 	    O11=O1
% 	),

% 	(cisC_prev(Name,PrevNode,O2,O2_)->
% 	 O22=O2_;
% 	    O22=O2
% 	),

% 	(cisC_prev(Name,PrevNode,O3,O3_)->
% 	 O33=O3_;
% 	    O33=O3
% 	),
% 	%genRefSet([O2,O3],O),
% 	data_facts:asserta_fact(def(Name-A,[O1])),
% 	data_facts:asserta_fact(ref(Name-A,[O2])),
% 	Literal=..[zext_rus,O11,O22,O33].


% interpret_bak(Name, Node,instr(_L,A,add,'3r',operands(O1,O2,O3)), Literal, _Ref, _Def, PrevNode):-
% 	(cisC(Name,Node,O1,O1_)->
% 	 O11=O1_;
% 	    O11=O1
% 	),

% 	(cisC_prev(Name,PrevNode,O2,O2_)->
% 	 O22=O2_;
% 	    O22=O2
% 	),

% 	(cisC_prev(Name,PrevNode,O3,O3_)->
% 	 O33=O3_;
% 	    O33=O3
% 	),
% 	genRefSet([O2,O3],O),
% 	data_facts:asserta_fact(def(Name-A,[O1])),
% 	data_facts:asserta_fact(ref(Name-A,O)),
% 	Literal=..[add_3r,O11,O22,O33].

% interpret_bak(Name, Node, calll(X, _CallCounter), Func,_,_,PrevNode):-
% 	(block_output_vars(X, OutputVars),
% 	 block_input_vars(X, InputVars),
% 	 getNewOutputVars(Name, Node, OutputVars, NewOutputVars),
% 	 getNewInputVars(Name, PrevNode, InputVars, NewInputVars),
% 	 append(NewInputVars, NewOutputVars, Arguments)
% 	->
% 	Func=..[X|Arguments];
% 	    Func=calll(X, _CallCounter)
% 	).

% interpret_bak(_Name, _Node,branch(X,Y),branch(X,Y),_,_,_).
% %interpret_bak(_Name, _Node,jmp(X),jmp(X),_,_,_).

% interpret_bak(Name, Node, jmp(X), Func,_,_,PrevNode):-
% 	(block_output_vars(X, OutputVars),
% 	 block_input_vars(X, InputVars),
% 	 getNewOutputVars(Name, Node, OutputVars, NewOutputVars),
% 	 getNewInputVars(Name, PrevNode, InputVars, NewInputVars),
% 	 append(NewInputVars, NewOutputVars, Arguments)
% 	->
% 	Func=..[X|Arguments];
% 	    Func=jmp(X)
% 	).

% % interpret_bak(_Name, _Node,instr(_L,_A,INSTR,F,_O), _Literal, _Ref, _Def, _PrevNode):-
% % 	nl,nl,nl,
% % 	error_message("~q, ~q",[INSTR,F]),nl,nl,nl, fail. 
getNewOutputVars(_Name, _Node, [], []).
getNewOutputVars(Name, Node, [V|Vs], [NewV|NewVs]):-
	cisC(Name, Node, V, NewV),
	getNewOutputVars(Name, Node, Vs, NewVs).
	
getNewInputVars(_Name, _Node, [], []).
getNewInputVars(Name, Node, [V|Vs], [NewV|NewVs]):-
	cisC_prev(Name, Node, V, NewV),
	getNewInputVars(Name, Node, Vs, NewVs).

genRefSet([],[]).
genRefSet([O|Operands],Set):-
 	(operandType(O,num)->
 	 genRefSet(Operands,Set);
 	    genRefSet(Operands,Set1),
 	    insert(Set1,O,Set)
 	 ).
%%%%%:- export(operandType/2).

operandType(O,num):-
	hex2decimal(O,_D).
operandType(O,num):-
	number(O).
%:- export(hex2decimal/2).
hex2decimal(H,D):-
 	atom_concat('0x',_N, H), 
 	atom_codes(_N,_Z), 
 	number_codes(D,16,_Z).

'3r'([add, and, eq, ld16s, ld8u, ldw, lss, lsu, or, shl, shr, sub, tsetr, ldaw]).

'l3r'([ashr, crc, divs, divu, lda16b, lda16f, ldawb, ldawf, mul, rems, remu, st16, st8, stw, xor]).
'l4r'([maccs]).

%'2rus'([addi, eqi, ldwi, shli, shri, stwi, subi]).
'2rus'([add, eq, ldw, shl, shr, stw, sub,ldaw]).
%'l2rus'([ashri, inpw, ldawbi, ldawfi, outpw]).
'l2rus'([ashr, inpw, ldawbi, ldawfi, outpw]).

%'ru6'([brbf, brbt, brff, brft, ldawdp, ldawsp, ldc, ldwcp, ldwdp, ldwsp, setci, stwdp, stwsp]).
'ru6'([bf, bt, ff, ft, ldaw, ldaw, ldc, ldw, ldw, ldw, set, stw, stw]).
%'lru6'([brbf, brbt, brff, brft, ldawdp, ldawsp, ldc, ldwcp, ldwdp, ldwsp, setci, stwdp, stwsp]).
'lru6'([bf, bt, ff, ft, ldaw, ldaw, ldc, ldw, ldw, ldw, set, stw, stw]).

%'u6'([blat, brbu, brfu, clrsr, entsp, extdp, extsp, getsr, kcalli, kentsp, krestsp, ldawcp, retsp, setsr]).
'u6'([blat, bu, brfu, clrsr, entsp, extdp, extsp, getsr, kcalli, kentsp, krestsp, ldaw, retsp, setsr]).
%'lu6'([blat, brbu, brfu, clrsr, entsp, extdp, extsp, getsr, kcalli, kentsp, krestsp, ldawcp, retsp, setsr]).
'lu6'([blat, bu, brfu, clrsr, entsp, extdp, extsp, getsr, kcalli, kentsp, krestsp, ldaw, retsp, setsr]).

%'u10'([blacp, blrb, blrf, ldapb, ldapf, ldwcpl]).
'u10'([blacp, bl, bl, ldapb, ldapf, ldwcpl]).
%'lu10'([blacp, blrb, blrf, ldapb, ldapf, ldwcpl]).
'lu10'([blacp, bl, bl, ldapb, ldapf, ldwcpl]).

'2r'([andnot, chkct, eef, eet, endin, getst, getts, in, inct, inshr, int, mkmsk, neg, not, outct, peek, sext, testct, testwct, 
	tinitcp, tinitdp, tinitpc, tinitsp, tsetmr, zext]).

%'r2r'([out, outshr, outt, setd, setpsc, setpt]).

%'l2r'([bitrev, byterev, clz, getd, getn, getps, setc, testlcl, tinitlr]).

%'lr2r'([setclk, setn, setps, setrdy, settw]).

%'rus'([chkcti, getr, mkmski, outcti, sexti, zexti]).
'rus'([chkcti, getr, mkmsk, outcti, sext, zext]).

instr_op(Instruction, Operands, Format, Expanded_op, KeepExpanded):-
	expand_op(Operands, Expanded_op, Reg_subscript),
	get_op_type(Expanded_op, Op_types),
	infer_format(Instruction, Op_types, Format),
	((Format='3r';Format='l3r';Reg_subscript>0)->
	  KeepExpanded=true
	 ;
	     KeepExpanded=false
	 ).

expand_op([],[],0).
expand_op([O|Rest], [O1, O2|OpRest],Reg_subscript):-
	separate_atom(O, O1, O2),!,
	(operand_type(O1, reg)->
	 RS1=1
	;
	    RS1=0
	),
	expand_op(Rest, OpRest,RS2),
	Reg_subscript is RS1 + RS2.
expand_op([O|Rest], [O|OpRest], Reg_subscript):-
	expand_op(Rest, OpRest, Reg_subscript).

get_op_type([],[]).
get_op_type([O|Rest], [OType|RestTypes]):-
	operand_type(O, OType),
	get_op_type(Rest, RestTypes).
get_op_type([O|Rest], RestTypes):-
	\+operand_type(O, _OType),
	get_op_type(Rest, RestTypes).


%:- export(operand_type/2).
operand_type(O, reg):-
	xs1_registers(Regs),
	member(O, Regs).
operand_type(O, (num,D)):-
	hex2decimal(O,D).
operand_type(O, (num,ON)):-
	atom_number(O, ON),
	number(ON).
operand_type(O, (label,O)):-
	atom_concat('.', _,O).

xs1_registers([r0,r1,r2,r3,r4,r5,r6,r7,r8,r9,r10,r11]).

infer_format(Instrunction, [reg, reg, reg, reg], 'l4r'):-
	'l4r'(I),
	member(Instrunction,I).

infer_format(Instrunction, [reg, reg, reg], '3r'):-
	'3r'(I),
	member(Instrunction,I). 

infer_format(Instrunction, [reg, reg, reg], 'l3r'):-
	'l3r'(I),
	member(Instrunction,I). 

infer_format(Instrunction, [reg, reg, (num, Value)], '2rus'):-
	Value < 12,
	'2rus'(I),
	member(Instrunction,I). 

infer_format(Instrunction, [reg, reg, (num, Value)], 'l2rus'):-
	Value < 12,
	'l2rus'(I),
	member(Instrunction,I). 

infer_format(bt, [reg], 'ru6'):-!. 
infer_format(bf, [reg], 'ru6'):-!. 


%6 bit integer
infer_format(Instrunction, [reg, (label, _Value)], 'ru6'):-
	% Value < 128,
	'ru6'(I),
	member(Instrunction,I),!. 

%16 bit integer
infer_format(Instrunction, [reg, (label, _Value)], 'lru6'):-
	'lru6'(I),
	member(Instrunction,I).

%6 bit integer
infer_format(Instrunction, [reg, (num, _Value)], 'rus'):-
%	Value < 128,
	'rus'(I),
	member(Instrunction,I),!.

%6 bit integer
infer_format(Instrunction, [reg, (num, Value)], 'ru6'):-
	Value < 128,
	'ru6'(I),
	member(Instrunction,I),!. 

%16 bit integer
infer_format(Instrunction, [reg, (num, _Value)], 'lru6'):-
	'lru6'(I),
	member(Instrunction,I). 

%6 bit integer , exceptions when bu is with label rather than the address.
infer_format(Instrunction, [(num, Value)], 'u6'):-
	Value< 128,
	'u6'(I),
	member(Instrunction,I). 

infer_format(bu, _Label, 'u6'):-!.

infer_format(Instrunction, [(num, _Value)], 'lu6'):-
	'lu6'(I),
	member(Instrunction,I).
% never reached.. Since we do not know the real addresses of each instruction.
infer_format(bu, _Label, 'lu6'):-!.

%10 bit integer 
infer_format(Instrunction, [(num, Value)], 'u10'):-
	Value < 2048,
	'u10'(I),
	member(Instrunction,I). 

infer_format(bl, _Label, 'u10'):-!.

%20 bit integer
infer_format(Instrunction, [(num, _Value)], 'lu10'):-
	'lu10'(I),
	member(Instrunction,I).


infer_format(Instrunction, [reg, reg], '2r'):-
	'2r'(I),
	member(Instrunction,I).

infer_format(Instrunction, [reg, (num, Value)], 'rus'):-
	Value< 12,
	'rus'(I),
	member(Instrunction,I).