:- module(tclp_aggregates_tr, _).

:- use_module(library(lists),
	    [
		append/3
	    ]).


:- dynamic 'trans$prolog'/1, 'trans$aggregate'/2.

do_term_expansion(0, _, _Module).

do_term_expansion(end_of_file, [end_of_file], _x) :- !.

do_term_expansion(':-'(Com), Clauses, _) :- !,
	expand_command(Com, Clauses).
%% 	expand_command(Com,Clauses),
%% 	display(Clauses), nl.

do_term_expansion(Clause, Clauses, _) :-
	(
	    Clause = (Head :- Body) ->
	    true
	;
	    (Clause :- true) = (Head :- Body)
	),
	functor(Head, Name, Arity),
	(
	    'trans$aggregate'(Name/Arity, AggModes) ->
	    convert_agg_clause(Head, Body, AggModes, Clauses)
	;
	    Clauses = Clause
	).
%% 	),
%% 	display(Clauses), nl.

convert_agg_clause(Head, Body, AggModes, (NewHead :- (PreBody, Body, nop(NewVars)))) :-
	Head =.. [Name|Args],
	length(Args, Arity),
	name(Name,CodeName),
	append("$", CodeName, NewCodeName),
	name(NewName, NewCodeName),
	length(NewVars, Arity),
	NewHead =.. [NewName|NewVars],
	convert_agg_clause_(Args,NewVars,AggModes,PreBodyList),
	list_to_conj(PreBodyList,PreBody).

convert_agg_clause_(_,_,[],[]).
convert_agg_clause_([A|Args],[A|NVars],['_'|Modes],Pres) :-	
	convert_agg_clause_(Args,NVars,Modes,Pres).
convert_agg_clause_([A|Args],[V|NVars],[M|Modes],[Pre|Pres]) :-
	Pre = (
		  get(V,(M,Att)),
		  Att = A
	      ),
	convert_agg_clause_(Args,NVars,Modes,Pres).
	
	
expand_command(table(Preds), Clauses) :- !,
	expand_command_table(Preds, Clauses, []).
	

expand_command_table((_Pred/_A, Preds), Clauses0, Clauses) :- !,
	expand_command_table(Preds, Clauses0, Clauses).
expand_command_table(_Pred/_A, Clauses, Clauses) :- !.
expand_command_table((Pred, Preds), Clauses0, Clauses) :-
	struct(Pred), !,
	Pred =.. [Name| AggModes], 
	length(AggModes, Arity),
	assert('trans$aggregate'(Name/Arity,AggModes)),
	expand_command_table_one(Name/Arity:AggModes, Clauses0, Clauses1),
	expand_command_table(Preds, Clauses1, Clauses).
expand_command_table(Pred, Clauses0, Clauses) :- 
	struct(Pred), !,
	Pred =.. [Name| AggModes],
	length(AggModes, Arity),
	assert('trans$aggregate'(Name/Arity,AggModes)),
	expand_command_table_one(Name/Arity:AggModes, Clauses0, Clauses).
expand_command_table(_, Clauses, Clauses).

expand_command_table_one(Name/Arity:AggModes, Clauses0, Clauses) :-
	change_entry_predicate(Name/Arity:AggModes, TabledDeclaration, NewClause),
	Clauses0 = [TabledDeclaration, NewClause|Clauses].


change_entry_predicate(Name/Arity:Modes, (:- table(NewName/Arity)), (Head :- (PreBody,NewHead,PostBody))):-
	length(Args, Arity),
	Head =.. [Name|Args],
	length(NewVars, Arity),
	name(Name,CodeName),
	append("$", CodeName, NewCodeName),
	name(NewName, NewCodeName),
	NewHead =.. [NewName|NewVars],
	change_entry_predicate_(Args,NewVars,Modes,PreBodyList,PostBodyList),
	list_to_conj(PreBodyList,PreBody),
	list_to_conj(PostBodyList,PostBody).

change_entry_predicate_(_Args,_NewVars,[],[],[]).
	
change_entry_predicate_([A|Args],[A|NVars],['_'|Modes],Pre,Post) :- !,
	change_entry_predicate_(Args,NVars,Modes,Pre,Post).
change_entry_predicate_([A|Args],[V|NVars],[M|Modes],[Pre|Pres], [Pos|Posts]) :-
	Pre = (put(V,(M,F))),
	Pos = (ground(A) -> entails(M,A,F) ; A=F ),
	change_entry_predicate_(Args,NVars,Modes,Pres,Posts).



conj_to_list(Term, List) :-
	conj_to_list_3(Term, List, []).
conj_to_list_3(Term, List0, List) :-
	( Term = (T1, T2) ->
	    conj_to_list_3(T1, List0, List1),
	    conj_to_list_3(T2, List1, List)
	; Term == true ->
	    List0 = List
	; List0 = [Term|List]
	).

list_to_conj([],         true).
list_to_conj([Lit|List], G0) :-
	( List == [] ->
	    G0 = Lit
	; G0 = (Lit, G),
	    list_to_conj(List, G)
	).
